@@ -4,8 +4,13 @@ import logging
 import random
 import socket
 import struct
+import sys
 import time
-from abc import ABCMeta, abstractmethod, abstractproperty
+from abc import ABCMeta, abstractmethod
+from binascii import hexlify
+
+from ipv8.messaging.deprecated.encoding import add_url_params
+from ipv8.taskmanager import TaskManager
 
 from libtorrent import bdecode
 
@@ -18,8 +23,6 @@ from twisted.python.failure import Failure
 from twisted.web.client import Agent, HTTPConnectionPool, RedirectAgent, readBody
 
 from Tribler.Core.Utilities.tracker_utils import parse_tracker_url
-from Tribler.pyipv8.ipv8.messaging.deprecated.encoding import add_url_params
-from Tribler.pyipv8.ipv8.taskmanager import TaskManager
 
 # Although these are the actions for UDP trackers, they can still be used as
 # identifiers.
@@ -30,16 +33,8 @@ TRACKER_ACTION_SCRAPE = 2
 MAX_INT32 = 2 ** 16 - 1
 
 UDP_TRACKER_INIT_CONNECTION_ID = 0x41727101980
-UDP_TRACKER_RECHECK_INTERVAL = 15
-UDP_TRACKER_MAX_RETRIES = 8
 
-HTTP_TRACKER_RECHECK_INTERVAL = 60
-HTTP_TRACKER_MAX_RETRIES = 0
-
-DHT_TRACKER_RECHECK_INTERVAL = 60
-DHT_TRACKER_MAX_RETRIES = 8
-
-MAX_TRACKER_MULTI_SCRAPE = 74
+MAX_INFOHASHES_IN_SCRAPE = 60
 
 
 def create_tracker_session(tracker_url, timeout, socket_manager, connection_pool=None):
@@ -73,7 +68,6 @@ class TrackerSession(TaskManager):
         self._infohash_list = []
         self.result_deferred = None
 
-        self._retries = 0
         self.timeout = timeout
         self.timeout_call = None
 
@@ -102,28 +96,18 @@ class TrackerSession(TaskManager):
         self.shutdown_task_manager()
         self._infohash_list = None
 
-    def can_add_request(self):
-        """
-        Checks if we still can add requests to this session.
-        :return: True or False.
-        """
-
-        #TODO(ardhi) : quickfix for etree.org can't handle multiple infohash in single call
-        etree_condition = "etree" not in self.tracker_url
-
-        return not self._is_initiated and len(self._infohash_list) < MAX_TRACKER_MULTI_SCRAPE and etree_condition
-
     def has_infohash(self, infohash):
         return infohash in self._infohash_list
 
     def add_infohash(self, infohash):
         """
-        Adds a infohash into this session.
+        Adds an infohash into this session.
         :param infohash: The infohash to be added.
         """
         assert not self._is_initiated, u"Must not add request to an initiated session."
         assert not self.has_infohash(infohash), u"Must not add duplicate requests"
-        self._infohash_list.append(infohash)
+        if len(self._infohash_list) < MAX_INFOHASHES_IN_SCRAPE:
+            self._infohash_list.append(infohash)
 
     @abstractmethod
     def connect_to_tracker(self):
@@ -146,16 +130,6 @@ class TrackerSession(TaskManager):
             self.result_deferred = None
         self.timeout_call = None
 
-    @abstractproperty
-    def max_retries(self):
-        """Number of retries before a session is marked as failed."""
-        pass
-
-    @abstractproperty
-    def retry_interval(self):
-        """Interval between retries."""
-        pass
-
     @property
     def tracker_type(self):
         return self._tracker_type
@@ -172,13 +146,6 @@ class TrackerSession(TaskManager):
     def last_contact(self):
         return self._last_contact
 
-    @property
-    def retries(self):
-        return self._retries
-
-    def increase_retries(self):
-        self._retries += 1
-
     @property
     def is_initiated(self):
         return self._is_initiated
@@ -209,20 +176,6 @@ class HttpTrackerSession(TrackerSession):
         self.request = None
         self._connection_pool = connection_pool if connection_pool else HTTPConnectionPool(reactor, False)
 
-    def max_retries(self):
-        """
-        Returns the max amount of retries allowed for this session.
-        :return: The maximum amount of retries.
-        """
-        return HTTP_TRACKER_MAX_RETRIES
-
-    def retry_interval(self):
-        """
-        Returns the interval one has to wait before retrying to connect.
-        :return: The interval before retrying.
-        """
-        return HTTP_TRACKER_RECHECK_INTERVAL
-
     def connect_to_tracker(self):
         # create the HTTP GET message
         # Note: some trackers have strange URLs, e.g.,
@@ -261,7 +214,6 @@ class HttpTrackerSession(TrackerSession):
         Handles the case of an error during the request.
         :param failure: The failure object that is thrown by a deferred.
         """
-        self._logger.info("Error when querying http tracker: %s %s", str(failure), self.tracker_url)
         self.failed(msg=failure.getErrorMessage())
 
     def on_response(self, response):
@@ -309,7 +261,7 @@ class HttpTrackerSession(TrackerSession):
             return
 
         response_dict = bdecode(body)
-        if response_dict is None:
+        if not response_dict:
             self.failed(msg="no valid response")
             return
 
@@ -327,7 +279,7 @@ class HttpTrackerSession(TrackerSession):
                 leechers = incomplete
 
                 # Store the information in the dictionary
-                response_list.append({'infohash': infohash.encode('hex'), 'seeders': seeders, 'leechers': leechers})
+                response_list.append({'infohash': hexlify(infohash), 'seeders': seeders, 'leechers': leechers})
 
                 # remove this infohash in the infohash list of this session
                 if infohash in unprocessed_infohash_list:
@@ -340,7 +292,7 @@ class HttpTrackerSession(TrackerSession):
 
         # handle the infohashes with no result (seeders/leechers = 0/0)
         for infohash in unprocessed_infohash_list:
-            response_list.append({'infohash': infohash.encode('hex'), 'seeders': 0, 'leechers': 0})
+            response_list.append({'infohash': hexlify(infohash), 'seeders': 0, 'leechers': 0})
 
         self._is_finished = True
         if self.result_deferred and not self.result_deferred.called:
@@ -423,7 +375,6 @@ class UdpTrackerSession(TrackerSession):
         Handles the case when resolving an ip address fails.
         :param failure: The failure object thrown by the deferred.
         """
-        self._logger.info("Error when querying UDP tracker: %s %s", str(failure), self.tracker_url)
         self.failed(msg=failure.getErrorMessage())
 
     def _on_cancel(self, _):
@@ -504,21 +455,6 @@ class UdpTrackerSession(TrackerSession):
         if self.timeout_call and self.timeout_call.active():
             self.timeout_call.cancel()
 
-    def max_retries(self):
-        """
-        Returns the max amount of retries allowed for this session.
-        :return: The maximum amount of retries.
-        """
-        return UDP_TRACKER_MAX_RETRIES
-
-    def retry_interval(self):
-        """
-        Returns the time one has to wait until retrying the connection again.
-        Increases exponentially with the number of retries.
-        :return: The interval one has to wait before retrying the connection.
-        """
-        return UDP_TRACKER_RECHECK_INTERVAL * (2 ** self._retries)
-
     def connect_to_tracker(self):
         """
         Connects to the tracker and starts querying for seed and leech data.
@@ -597,8 +533,13 @@ class UdpTrackerSession(TrackerSession):
         self.generate_transaction_id()
 
         # pack and send the message
+        if sys.version_info.major > 2:
+            infohash_list = self._infohash_list
+        else:
+            infohash_list = [str(infohash) for infohash in self._infohash_list]
+
         fmt = '!qii' + ('20s' * len(self._infohash_list))
-        message = struct.pack(fmt, self._connection_id, self.action, self.transaction_id, *self._infohash_list)
+        message = struct.pack(fmt, self._connection_id, self.action, self.transaction_id, *infohash_list)
 
         # Send the scrape message
         self.socket_mgr.send_request(message, self)
@@ -645,7 +586,7 @@ class UdpTrackerSession(TrackerSession):
             # Store the information in the hash dict to be returned.
             # Sow complete as seeders. "complete: number of peers with the entire file, i.e. seeders (integer)"
             #  - https://wiki.theory.org/BitTorrentSpecification#Tracker_.27scrape.27_Convention
-            response_list.append({'infohash': infohash.encode('hex'), 'seeders': complete, 'leechers': incomplete})
+            response_list.append({'infohash': hexlify(infohash), 'seeders': complete, 'leechers': incomplete})
 
         # close this socket and remove its transaction ID from the list
         self.remove_transaction_id()
@@ -677,13 +618,6 @@ class FakeDHTSession(TrackerSession):
         # Return a defer that immediately calls its callback
         return defer.succeed(None)
 
-    def can_add_request(self):
-        """
-        Returns whether or not this session can accept additional infohashes.
-        :return:
-        """
-        return True
-
     def add_infohash(self, infohash):
         """
         This function adds a infohash to the request list.
@@ -694,38 +628,33 @@ class FakeDHTSession(TrackerSession):
     def connect_to_tracker(self):
         """
         Fakely connects to a tracker.
-        :return: A deferred with a callback containing an empty dictionary.
+        :return: A deferred that fires with the health information.
         """
-        def on_metainfo_received(metainfo):
-            self.result_deferred.callback({'DHT': [{'infohash': self.infohash.encode('hex'),
-                                                    'seeders': metainfo['seeders'], 'leechers': metainfo['leechers']}]})
+        def on_metainfo(metainfo):
+            if not metainfo:
+                self.result_deferred.errback(Failure(RuntimeError("Metainfo lookup error")))
+                return
 
-        def on_metainfo_timeout(_):
-            self.result_deferred.errback(Failure(RuntimeError("DHT timeout")))
-
-        if self._session:
-            self._session.lm.ltmgr.get_metainfo(self.infohash, callback=on_metainfo_received,
-                                                timeout_callback=on_metainfo_timeout, timeout=self.timeout)
+            self.result_deferred.callback({
+                "DHT": [{
+                    "infohash": hexlify(self.infohash),
+                    "seeders": metainfo["seeders"],
+                    "leechers": metainfo["leechers"]
+                }]
+            })
 
+        self._session.lm.ltmgr.get_metainfo(self.infohash, timeout=self.timeout).addCallback(on_metainfo)
         return self.result_deferred
 
-    @property
-    def max_retries(self):
-        """
-        Returns the max amount of retries allowed for this session.
-        :return: The maximum amount of retries.
-        """
-        return DHT_TRACKER_MAX_RETRIES
 
-    @property
-    def retry_interval(self):
-        """
-        Returns the interval one has to wait before retrying to connect.
-        :return: The interval before retrying.
-        """
-        return DHT_TRACKER_RECHECK_INTERVAL
+class FakeBep33DHTSession(FakeDHTSession):
+    """
+    Fake session for a BEP33 lookup.
+    """
 
-    @property
-    def last_contact(self):
-        # we never want this session to be cleaned up as it's faker than a 4 eur bill.
-        return time.time()
+    def connect_to_tracker(self):
+        """
+        Fakely connects to a tracker.
+        :return: A deferred that fires with the health information.
+        """
+        return self._session.lm.ltmgr.dht_health_manager.get_health(bytes(self.infohash))
