@@ -1,8 +1,11 @@
+from __future__ import absolute_import
+
 import os
 
 from configobj import ConfigObj
 
-from Tribler.Core.Config.tribler_config import TriblerConfig, CONFIG_SPEC_PATH, FILENAME
+from Tribler.Core.Config.tribler_config import CONFIG_FILENAME, CONFIG_SPEC_PATH, TriblerConfig
+from Tribler.Core.osutils import get_home_dir
 from Tribler.Test.Core.base_test import TriblerCoreTest
 
 
@@ -17,9 +20,10 @@ class TestTriblerConfig(TriblerCoreTest):
         """
         super(TestTriblerConfig, self).setUp()
 
+        state_dir = self.getStateDir()
+        TriblerConfig.get_default_state_dir = lambda _: state_dir
+
         self.tribler_config = TriblerConfig()
-        self.tribler_config.get_default_state_dir = lambda **_: self.getStateDir()
-        self.tribler_config.set_state_dir(self.getStateDir())
         self.assertIsNotNone(self.tribler_config)
 
     def test_init_with_config(self):
@@ -47,7 +51,7 @@ class TestTriblerConfig(TriblerCoreTest):
         port = 4444
         self.tribler_config.set_anon_listen_port(port)
         self.tribler_config.write()
-        path = os.path.join(self.tribler_config.get_default_state_dir(), FILENAME)
+        path = os.path.join(self.tribler_config.get_default_state_dir(), CONFIG_FILENAME)
         read_config = TriblerConfig.load(path)
 
         read_config.validate()
@@ -98,35 +102,49 @@ class TestTriblerConfig(TriblerCoreTest):
         self.tribler_config.set_credit_mining_sources(source_list)
         self.assertEqual(self.tribler_config.get_credit_mining_sources(), source_list)
 
+    def test_bootstrap_configs(self):
+
+        self.tribler_config.set_bootstrap_enabled(False)
+        self.assertFalse(self.tribler_config.get_bootstrap_enabled())
+
+        self.tribler_config.set_bootstrap_max_download_rate(20)
+        self.assertEqual(self.tribler_config.get_bootstrap_max_download_rate(), 20)
+
+        self.tribler_config.set_bootstrap_infohash("TestInfohash")
+        self.assertEqual(self.tribler_config.get_bootstrap_infohash(), "TestInfohash")
+
+    def test_relative_paths(self):
+        # Default should be taken from config.spec
+        self.assertEqual(self.tribler_config.get_trustchain_keypair_filename(),
+                         os.path.abspath(os.path.join(self.state_dir, "ec_multichain.pem")))
+
+        local_name = os.path.join("somedir", "ec_multichain.pem")
+        global_name = os.path.join(self.state_dir, local_name)
+        self.tribler_config.set_trustchain_keypair_filename(global_name)
+
+        # It should always return global path
+        self.assertEqual(self.tribler_config.get_trustchain_keypair_filename(), global_name)
+        # But internally it should be stored as a local path
+        self.assertEqual(self.tribler_config.config['trustchain']['ec_keypair_filename'], local_name)
+
+        # If it points out of the state dir, it should be saved as a global path
+        out_of_dir_name_global = os.path.abspath(os.path.join(self.state_dir, "..", "filename"))
+        self.tribler_config.set_trustchain_keypair_filename(out_of_dir_name_global)
+        self.assertEqual(self.tribler_config.config['trustchain']['ec_keypair_filename'], out_of_dir_name_global)
+
+
     def test_get_set_methods_general(self):
         """
         Check whether general get and set methods are working as expected.
         """
-        self.tribler_config.set_family_filter_enabled(False)
-        self.assertEqual(self.tribler_config.get_family_filter_enabled(), False)
-
         self.tribler_config.set_state_dir("TEST")
         self.assertEqual(self.tribler_config.get_state_dir(), "TEST")
 
-        self.tribler_config.set_permid_keypair_filename(None)
-        self.assertEqual(self.tribler_config.get_permid_keypair_filename(), os.path.join("TEST", "ec.pem"))
-        self.tribler_config.set_permid_keypair_filename("TEST")
-        self.assertEqual(self.tribler_config.get_permid_keypair_filename(), "TEST")
-
-        self.tribler_config.set_trustchain_keypair_filename(None)
-        self.assertEqual(self.tribler_config.get_trustchain_keypair_filename(),
-                         os.path.join("TEST", "ec_multichain.pem"))
-        self.tribler_config.set_trustchain_keypair_filename("TEST")
-        self.assertEqual(self.tribler_config.get_trustchain_keypair_filename(), "TEST")
-
-        self.tribler_config.set_trustchain_testnet_keypair_filename(None)
         self.assertEqual(self.tribler_config.get_trustchain_testnet_keypair_filename(),
                          os.path.join("TEST", "ec_trustchain_testnet.pem"))
-        self.tribler_config.set_trustchain_testnet_keypair_filename("TEST")
-        self.assertEqual(self.tribler_config.get_trustchain_testnet_keypair_filename(), "TEST")
-
-        self.tribler_config.set_megacache_enabled(True)
-        self.assertEqual(self.tribler_config.get_megacache_enabled(), True)
+        self.tribler_config.set_trustchain_testnet_keypair_filename("bla2")
+        self.assertEqual(self.tribler_config.get_trustchain_testnet_keypair_filename(),
+                         os.path.join("TEST", "bla2"))
 
         self.tribler_config.set_testnet(True)
         self.assertTrue(self.tribler_config.get_testnet())
@@ -162,21 +180,14 @@ class TestTriblerConfig(TriblerCoreTest):
         self.tribler_config.set_http_api_retry_port(True)
         self.assertTrue(self.tribler_config.get_http_api_retry_port())
 
-    def test_get_set_methods_dispersy(self):
-        """
-        Check whether dispersy get and set methods are working as expected.
-        """
-        self.tribler_config.set_dispersy_enabled(True)
-        self.assertEqual(self.tribler_config.get_dispersy_enabled(), True)
-        self.tribler_config.set_dispersy_port(True)
-        self.assertEqual(self.tribler_config.get_dispersy_port(), True)
-
     def test_get_set_methods_ipv8(self):
         """
         Check whether IPv8 get and set methods are working as expected.
         """
         self.tribler_config.set_ipv8_enabled(False)
         self.assertEqual(self.tribler_config.get_ipv8_enabled(), False)
+        self.tribler_config.set_ipv8_port(1234)
+        self.assertEqual(self.tribler_config.get_ipv8_port(), 1234)
         self.tribler_config.set_ipv8_bootstrap_override("127.0.0.1:12345")
         self.assertEqual(self.tribler_config.get_ipv8_bootstrap_override(), ("127.0.0.1", 12345))
         self.tribler_config.set_ipv8_statistics(True)
@@ -212,15 +223,6 @@ class TestTriblerConfig(TriblerCoreTest):
         self.tribler_config.set_libtorrent_dht_enabled(False)
         self.assertFalse(self.tribler_config.get_libtorrent_dht_enabled())
 
-    def test_get_set_methods_mainline_dht(self):
-        """
-        Check whether mainline dht get and set methods are working as expected.
-        """
-        self.tribler_config.set_mainline_dht_enabled(True)
-        self.assertEqual(self.tribler_config.get_mainline_dht_enabled(), True)
-        self.tribler_config.set_mainline_dht_port(True)
-        self.assertEqual(self.tribler_config.get_mainline_dht_port(), True)
-
     def test_get_set_methods_video_server(self):
         """
         Check whether video server get and set methods are working as expected.
@@ -248,23 +250,13 @@ class TestTriblerConfig(TriblerCoreTest):
         self.assertEqual(self.tribler_config.get_default_anonymity_enabled(), True)
         self.tribler_config.set_default_safeseeding_enabled(True)
         self.assertEqual(self.tribler_config.get_default_safeseeding_enabled(), True)
-        self.tribler_config.set_default_destination_dir(True)
-        self.assertEqual(self.tribler_config.get_default_destination_dir(), True)
+        self.tribler_config.set_default_destination_dir(get_home_dir())
+        self.assertEqual(self.tribler_config.get_default_destination_dir(), get_home_dir())
         self.tribler_config.set_tunnel_community_random_slots(10)
         self.assertEqual(self.tribler_config.get_tunnel_community_random_slots(), 10)
         self.tribler_config.set_tunnel_community_competing_slots(20)
         self.assertEqual(self.tribler_config.get_tunnel_community_competing_slots(), 20)
 
-    def test_get_set_methods_torrent_store(self):
-        """
-        Check whether torrent store get and set methods are working as expected.
-        """
-        self.tribler_config.set_torrent_store_enabled(True)
-        self.assertEqual(self.tribler_config.get_torrent_store_enabled(), True)
-        self.tribler_config.set_torrent_store_dir("TESTDIR")
-        self.tribler_config.set_state_dir("TEST")
-        self.assertEqual(self.tribler_config.get_torrent_store_dir(), os.path.join("TEST", "TESTDIR"))
-
     def test_get_set_methods_wallets(self):
         """
         Check whether wallet get and set methods are working as expected.
@@ -274,6 +266,16 @@ class TestTriblerConfig(TriblerCoreTest):
         self.tribler_config.set_bitcoinlib_enabled(False)
         self.assertFalse(self.tribler_config.get_bitcoinlib_enabled())
 
+    def test_get_set_chant_methods(self):
+        """
+        Check whether chant get and set methods are working as expected.
+        """
+        self.tribler_config.set_chant_enabled(False)
+        self.assertFalse(self.tribler_config.get_chant_enabled())
+        self.tribler_config.set_chant_channels_dir('test')
+        self.assertEqual(self.tribler_config.get_chant_channels_dir(),
+                         os.path.join(self.tribler_config.get_state_dir(), 'test'))
+
     def test_get_set_is_matchmaker(self):
         """
         Check whether matchmaker get and set methods are working as expected.
@@ -281,55 +283,6 @@ class TestTriblerConfig(TriblerCoreTest):
         self.tribler_config.set_is_matchmaker(False)
         self.assertFalse(self.tribler_config.get_is_matchmaker())
 
-    def test_get_set_methods_metadata(self):
-        """
-        Check whether metadata get and set methods are working as expected.
-        """
-        self.tribler_config.set_metadata_enabled(True)
-        self.assertEqual(self.tribler_config.get_metadata_enabled(), True)
-        self.tribler_config.set_metadata_store_dir("TESTDIR")
-        self.tribler_config.set_state_dir("TEST")
-        self.assertEqual(self.tribler_config.get_metadata_store_dir(), os.path.join("TEST", "TESTDIR"))
-
-    def test_get_set_methods_torrent_collecting(self):
-        """
-        Check whether torrent collecting get and set methods are working as expected.
-        """
-        self.tribler_config.set_torrent_collecting_enabled(True)
-        self.assertEqual(self.tribler_config.get_torrent_collecting_enabled(), True)
-        self.tribler_config.set_torrent_collecting_max_torrents(True)
-        self.assertEqual(self.tribler_config.get_torrent_collecting_max_torrents(), True)
-        self.tribler_config.set_torrent_collecting_dir(True)
-        self.assertEqual(self.tribler_config.get_torrent_collecting_dir(), True)
-
-    def test_get_set_methods_search_community(self):
-        """
-        Check whether search community get and set methods are working as expected.
-        """
-        self.tribler_config.set_torrent_search_enabled(True)
-        self.assertEqual(self.tribler_config.get_torrent_search_enabled(), True)
-
-    def test_get_set_methods_allchannel_community(self):
-        """
-        Check whether allchannel community get and set methods are working as expected.
-        """
-        self.tribler_config.set_channel_search_enabled(True)
-        self.assertEqual(self.tribler_config.get_channel_search_enabled(), True)
-
-    def test_get_set_methods_channel_community(self):
-        """
-        Check whether channel community get and set methods are working as expected.
-        """
-        self.tribler_config.set_channel_community_enabled(True)
-        self.assertEqual(self.tribler_config.get_channel_community_enabled(), True)
-
-    def test_get_set_methods_preview_channel_community(self):
-        """
-        Check whether preview channel community get and set methods are working as expected.
-        """
-        self.tribler_config.set_preview_channel_community_enabled(True)
-        self.assertEqual(self.tribler_config.get_preview_channel_community_enabled(), True)
-
     def test_get_set_methods_popularity_community(self):
         """
         Check whether popularity community get and set methods are working as expected.
@@ -343,8 +296,8 @@ class TestTriblerConfig(TriblerCoreTest):
         """
         self.tribler_config.set_watch_folder_enabled(True)
         self.assertEqual(self.tribler_config.get_watch_folder_enabled(), True)
-        self.tribler_config.set_watch_folder_path(True)
-        self.assertEqual(self.tribler_config.get_watch_folder_path(), True)
+        self.tribler_config.set_watch_folder_path(get_home_dir())
+        self.assertEqual(self.tribler_config.get_watch_folder_path(), get_home_dir())
 
     def test_get_set_methods_resource_monitor(self):
         """
@@ -386,3 +339,11 @@ class TestTriblerConfig(TriblerCoreTest):
         """
         self.tribler_config.set_record_transactions(True)
         self.assertTrue(self.tribler_config.get_record_transactions())
+
+    def test_get_set_default_add_download_to_channel(self):
+        """
+        Check whether set/get methods of default add download to channel works.
+        """
+        self.assertEqual(self.tribler_config.get_default_add_download_to_channel(), False)
+        self.tribler_config.set_default_add_download_to_channel(True)
+        self.assertEqual(self.tribler_config.get_default_add_download_to_channel(), True)
