@@ -1,36 +1,42 @@
+from __future__ import absolute_import
+
 import logging
 import os
+import subprocess
 import sys
 import threading
-from random import randint
-import unittest
-from unittest import skipUnless, skipIf
-
 import time
-from PyQt5.QtCore import QPoint, Qt, QTimer
+from unittest import TestCase, skipIf, skipUnless
+
+from PyQt5.QtCore import QPoint, QTimer, Qt
 from PyQt5.QtGui import QPixmap, QRegion
 from PyQt5.QtTest import QTest
-from PyQt5.QtWidgets import QApplication, QListWidget, QTreeWidget, QTextEdit
+from PyQt5.QtWidgets import QApplication, QListWidget, QTextEdit, QTreeWidget
+from PyQt5.QtWidgets import QTableView
+
+from six import text_type
+from six.moves import xrange
 
-import TriblerGUI.core_manager as core_manager
-import TriblerGUI.defs
 from Tribler.Core.Utilities.network_utils import get_random_port
-from TriblerGUI.dialogs.feedbackdialog import FeedbackDialog
-from TriblerGUI.widgets.channel_torrent_list_item import ChannelTorrentListItem
-from TriblerGUI.widgets.home_recommended_item import HomeRecommendedItem
-
-api_port = get_random_port()
-core_manager.START_FAKE_API = True
+from Tribler.Test.common import TORRENT_UBUNTU_FILE
 
 import TriblerGUI
-TriblerGUI.defs.DEFAULT_API_PORT = api_port
-
-from TriblerGUI.widgets.loading_list_item import LoadingListItem
+import TriblerGUI.core_manager as core_manager
+import TriblerGUI.defs
+from TriblerGUI.dialogs.feedbackdialog import FeedbackDialog
+from TriblerGUI.tribler_app import TriblerApplication
 from TriblerGUI.tribler_window import TriblerWindow
+from TriblerGUI.widgets.home_recommended_item import HomeRecommendedItem
+from TriblerGUI.widgets.loading_list_item import LoadingListItem
 
 if os.environ.get("TEST_GUI") == "yes":
-    app = QApplication(sys.argv)
+    api_port = get_random_port()
+    core_manager.START_FAKE_API = True
+    TriblerGUI.defs.DEFAULT_API_PORT = api_port
+
+    app = TriblerApplication("triblerapp-guitest", sys.argv)
     window = TriblerWindow(api_port=api_port)
+    app.set_activation_window(window)
     QTest.qWaitForWindowExposed(window)
 else:
     window = None
@@ -40,10 +46,10 @@ def start_fake_core(port):
     from twisted.internet import reactor
     from twisted.web.server import Site
 
-    from FakeTriblerAPI.endpoints.root_endpoint import RootEndpoint
-    from FakeTriblerAPI.endpoints.video_root_endpoint import VideoRootEndpoint
-    from FakeTriblerAPI.tribler_data import TriblerData
-    import FakeTriblerAPI.tribler_utils as tribler_utils
+    from Tribler.Test.GUI.FakeTriblerAPI.endpoints.root_endpoint import RootEndpoint
+    from Tribler.Test.GUI.FakeTriblerAPI.endpoints.video_root_endpoint import VideoRootEndpoint
+    from Tribler.Test.GUI.FakeTriblerAPI.tribler_data import TriblerData
+    import Tribler.Test.GUI.FakeTriblerAPI.tribler_utils as tribler_utils
 
     def generate_tribler_data():
         tribler_utils.tribler_data = TriblerData()
@@ -67,22 +73,26 @@ def start_fake_core(port):
     reactor.run(installSignalHandlers=False)
 
 
-# Start the fake API
-t = threading.Thread(target=start_fake_core, args=(api_port,))
-t.setDaemon(True)
-t.start()
+if os.environ.get("TEST_GUI") == "yes":
+    # Start the fake API
+    t = threading.Thread(target=start_fake_core, args=(api_port,))
+    t.setDaemon(True)
+    t.start()
 
 
 def no_abort(*args, **kwargs):
     sys.__excepthook__(*args, **kwargs)
-sys.excepthook = no_abort
+
+
+if os.environ.get("TEST_GUI") == "yes":
+    sys.excepthook = no_abort
 
 
 class TimeoutException(Exception):
     pass
 
 
-class AbstractTriblerGUITest(unittest.TestCase):
+class AbstractTriblerGUITest(TestCase):
     """
     This class contains various utility methods that are used during the GUI test, i.e. methods that wait until
     some data in a list is loaded or for taking a screenshot of the current window.
@@ -139,6 +149,8 @@ class AbstractTriblerGUITest(unittest.TestCase):
             elif isinstance(llist, QTreeWidget) and llist.topLevelItemCount() >= num_items:
                 if not isinstance(llist.topLevelItem(0), LoadingListItem):
                     return
+            elif isinstance(llist, QTableView) and llist.verticalHeader().count() >= num_items:
+                return
 
         # List was not populated in time, fail the test
         raise TimeoutException("The list was not populated within 10 seconds")
@@ -160,6 +172,13 @@ class AbstractTriblerGUITest(unittest.TestCase):
 
         raise TimeoutException("Did not receive settings within 10 seconds")
 
+    def wait_for_something(self, something, timeout=10):
+        for _ in range(0, timeout * 1000, 100):
+            QTest.qWait(100)
+            if something is not None:
+                return
+        raise TimeoutException("The value was not set within 10 seconds")
+
     def get_attr_recursive(self, attr_name):
         parts = attr_name.split(".")
         cur_attr = window
@@ -204,6 +223,11 @@ class AbstractTriblerGUITest(unittest.TestCase):
         # QTextEdit was not populated in time, fail the test
         raise TimeoutException("QTextEdit was not populated within 10 seconds")
 
+    def get_index_of_row(self, table_view, row):
+        x = table_view.columnViewportPosition(0)
+        y = table_view.rowViewportPosition(row)
+        return table_view.indexAt(QPoint(x, y))
+
 
 @skipUnless(os.environ.get("TEST_GUI") == "yes", "Not testing the GUI by default")
 class TriblerGUITest(AbstractTriblerGUITest):
@@ -211,6 +235,35 @@ class TriblerGUITest(AbstractTriblerGUITest):
     GUI tests for the GUI written in PyQt. These methods are using the QTest framework to simulate mouse clicks.
     """
 
+    @skipIf(sys.platform == "win32", "This test is unreliable on Windows")
+    def test_run_tribler(self):
+        """
+        Tests running a second instance of Tribler with a torrent file. Simulates user clicking on a Ubuntu torrent
+        file.
+        """
+        def on_app_message(msg):
+            self.assertEqual(msg, "file:%s" % TORRENT_UBUNTU_FILE)
+
+        app.messageReceived.connect(on_app_message)
+
+        # Start a second Tribler instance with a torrent file
+        system_encoding = sys.getfilesystemencoding()
+        test_env = {(k.encode(system_encoding) if isinstance(k, text_type) else str(k))
+                    : (v.encode(system_encoding) if isinstance(v, text_type) else str(v))
+                    for k, v in os.environ.copy().items()}
+        test_env['TRIBLER_APP_NAME'] = 'triblerapp-guitest'
+
+        tribler_executable = os.path.join(os.path.dirname(os.path.dirname(TriblerGUI.__file__)), "run_tribler.py")
+        tribler_instance_2 = subprocess.Popen(['python', tribler_executable, TORRENT_UBUNTU_FILE], env=test_env)
+        _process_stream = tribler_instance_2.communicate()[0]
+        self.assertEqual(tribler_instance_2.returncode, 1)
+
+        QTest.qWait(200)
+
+        torrent_name_in_dialog = window.dialog.dialog_widget.torrent_name_label.text()
+        self.assertEqual(torrent_name_in_dialog, TORRENT_UBUNTU_FILE)
+        self.screenshot(window, name="start_download_dialog_on_startup")
+
     def test_home_page_torrents(self):
         QTest.mouseClick(window.left_menu_button_home, Qt.LeftButton)
         QTest.mouseClick(window.home_tab_torrents_button, Qt.LeftButton)
@@ -231,12 +284,64 @@ class TriblerGUITest(AbstractTriblerGUITest):
         self.wait_for_list_populated(window.subscribed_channels_list)
         self.screenshot(window, name="subscriptions")
 
-        first_widget = window.subscribed_channels_list.itemWidget(window.subscribed_channels_list.item(0))
-        QTest.mouseClick(first_widget.subscribe_button, Qt.LeftButton)
-        self.wait_for_signal(first_widget.subscriptions_widget.unsubscribed_channel)
+        # Sort
+        window.subscribed_channels_list.sortByColumn(1, 1)
+        self.wait_for_list_populated(window.subscribed_channels_list)
+        self.screenshot(window, name="subscriptions_sorted")
+        self.assertLessEqual(window.subscribed_channels_list.verticalHeader().count(), 50)
+
+        # Filter
+        old_num_items = window.subscribed_channels_list.verticalHeader().count()
+        QTest.keyClick(window.subscribed_channels_filter_input, '1')
+        self.wait_for_list_populated(window.subscribed_channels_list)
+        self.screenshot(window, name="subscriptions_filtered")
+        self.assertLessEqual(window.subscribed_channels_list.verticalHeader().count(), old_num_items)
+        window.subscribed_channels_filter_input.setText('')
+        self.wait_for_list_populated(window.subscribed_channels_list)
+
+        # Unsubscribe and subscribe again
+        index = self.get_index_of_row(window.subscribed_channels_list, 0)
+        window.subscribed_channels_list.on_subscribe_control_clicked(index)
+        QTest.qWait(200)
         self.screenshot(window, name="unsubscribed")
-        QTest.mouseClick(first_widget.subscribe_button, Qt.LeftButton)
-        self.wait_for_signal(first_widget.subscriptions_widget.subscribed_channel)
+        window.subscribed_channels_list.on_subscribe_control_clicked(index)
+        QTest.qWait(200)
+
+    def test_discovered_page(self):
+        QTest.mouseClick(window.left_menu_button_discovered, Qt.LeftButton)
+        self.wait_for_list_populated(window.discovered_channels_list)
+        self.screenshot(window, name="discovered_page")
+
+        # Sort
+        window.discovered_channels_list.sortByColumn(1, 1)
+        self.wait_for_list_populated(window.discovered_channels_list)
+        self.screenshot(window, name="discovered_sorted")
+        self.assertLessEqual(window.discovered_channels_list.verticalHeader().count(), 50)
+
+        # Filter
+        old_num_items = window.discovered_channels_list.verticalHeader().count()
+        QTest.keyClick(window.discovered_channels_filter_input, '1')
+        self.wait_for_list_populated(window.discovered_channels_list)
+        self.screenshot(window, name="discovered_filtered")
+        self.assertLessEqual(window.discovered_channels_list.verticalHeader().count(), old_num_items)
+
+    def test_channel_torrents(self):
+        QTest.mouseClick(window.left_menu_button_subscriptions, Qt.LeftButton)
+        self.wait_for_list_populated(window.subscribed_channels_list)
+        index = self.get_index_of_row(window.subscribed_channels_list, 0)
+        window.subscribed_channels_list.on_table_item_clicked(index)
+        self.wait_for_list_populated(window.channel_page_container.content_table)
+        self.screenshot(window, name="channel_torrents_loaded")
+
+        # Toggle credit mining
+        QTest.mouseClick(window.credit_mining_button, Qt.LeftButton)
+        self.wait_for_signal(window.subscription_widget.credit_mining_toggled)
+
+        # Click the first torrent
+        index = self.get_index_of_row(window.channel_page_container.content_table, 0)
+        window.channel_page_container.content_table.on_table_item_clicked(index)
+        QTest.qWait(100)
+        self.screenshot(window, name="channel_overview_details")
 
     def test_edit_channel_overview(self):
         QTest.mouseClick(window.left_menu_button_my_channel, Qt.LeftButton)
@@ -256,53 +361,30 @@ class TriblerGUITest(AbstractTriblerGUITest):
         self.wait_for_variable("edit_channel_page.channel_overview")
         QTest.mouseClick(window.edit_channel_torrents_button, Qt.LeftButton)
         self.screenshot(window, name="edit_channel_torrents_loading")
-        self.wait_for_list_populated(window.edit_channel_torrents_list)
+        self.wait_for_list_populated(window.edit_channel_torrents_container.content_table)
         self.screenshot(window, name="edit_channel_torrents")
 
-        first_widget = window.edit_channel_torrents_list.itemWidget(window.edit_channel_torrents_list.item(0))
-        QTest.mouseClick(first_widget, Qt.LeftButton)
-        self.screenshot(window, name="edit_channel_torrents_selected")
-        QTest.mouseClick(window.edit_channel_torrents_remove_selected_button, Qt.LeftButton)
-        self.screenshot(window, name="remove_channel_torrent_dialog")
-        QTest.mouseClick(window.edit_channel_page.dialog.buttons[1], Qt.LeftButton)
+        # Sort
+        window.edit_channel_torrents_container.content_table.sortByColumn(2, 1)  # Size
+        self.wait_for_list_populated(window.edit_channel_torrents_container.content_table)
+        self.screenshot(window, name="edit_channel_torrents_sorted")
+        self.wait_for_something(window.edit_channel_torrents_container.content_table.model().total_items)
+        self.assertLessEqual(window.discovered_channels_list.verticalHeader().count(), 50)
 
-        QTest.mouseClick(window.edit_channel_torrents_remove_all_button, Qt.LeftButton)
-        self.screenshot(window, name="remove_all_channel_torrent_dialog")
-        QTest.mouseClick(window.edit_channel_page.dialog.buttons[1], Qt.LeftButton)
+        # Filter
+        old_num_items = window.edit_channel_torrents_container.content_table.verticalHeader().count()
+        QTest.keyClick(window.edit_channel_torrents_filter, 'a')
+        self.wait_for_list_populated(window.edit_channel_torrents_container.content_table)
+        self.screenshot(window, name="edit_channel_torrents_filtered")
+        self.assertLessEqual(window.edit_channel_torrents_container.content_table.verticalHeader().count(),
+                             old_num_items)
+        window.edit_channel_torrents_filter.setText('')
+        self.wait_for_list_populated(window.edit_channel_torrents_container.content_table)
 
-    def test_edit_channel_playlists(self):
-        QTest.mouseClick(window.left_menu_button_my_channel, Qt.LeftButton)
-        self.wait_for_variable("edit_channel_page.channel_overview")
-        QTest.mouseClick(window.edit_channel_playlists_button, Qt.LeftButton)
-        self.screenshot(window, name="edit_channel_playlists_loading")
-        self.wait_for_list_populated(window.edit_channel_playlists_list)
-        self.screenshot(window, name="edit_channel_playlists")
-
-    def test_edit_channel_rssfeeds(self):
-        QTest.mouseClick(window.left_menu_button_my_channel, Qt.LeftButton)
-        self.wait_for_variable("edit_channel_page.channel_overview")
-        QTest.mouseClick(window.edit_channel_rss_feeds_button, Qt.LeftButton)
-        self.screenshot(window, name="edit_channel_rssfeeds_loading")
-        self.wait_for_list_populated(window.edit_channel_rss_feeds_list)
-        self.screenshot(window, name="edit_channel_rssfeeds")
-
-    def test_add_remove_refresh_rssfeed(self):
-        QTest.mouseClick(window.left_menu_button_my_channel, Qt.LeftButton)
-        self.wait_for_variable("edit_channel_page.channel_overview")
-        QTest.mouseClick(window.edit_channel_rss_feeds_button, Qt.LeftButton)
-        self.wait_for_list_populated(window.edit_channel_rss_feeds_list)
-        QTest.mouseClick(window.edit_channel_details_rss_add_button, Qt.LeftButton)
-        self.screenshot(window, name="edit_channel_add_rssfeeds_dialog")
-        window.edit_channel_page.dialog.dialog_widget.dialog_input.setText("http://test.com/rss.xml")
-        QTest.mouseClick(window.edit_channel_page.dialog.buttons[0], Qt.LeftButton)
-
-        # Remove item
-        window.edit_channel_rss_feeds_list.topLevelItem(0).setSelected(True)
-        QTest.mouseClick(window.edit_channel_details_rss_feeds_remove_selected_button, Qt.LeftButton)
-        self.screenshot(window, name="edit_channel_remove_rssfeeds_dialog")
-        QTest.mouseClick(window.edit_channel_page.dialog.buttons[0], Qt.LeftButton)
-
-        QTest.mouseClick(window.edit_channel_details_rss_refresh_button, Qt.LeftButton)
+        # Commit the result
+        QTest.mouseClick(window.edit_channel_commit_button, Qt.LeftButton)
+        self.wait_for_signal(window.edit_channel_page.on_commit, no_args=True)
+        self.screenshot(window, name="edit_channel_committed")
 
     def test_settings(self):
         QTest.mouseClick(window.settings_button, Qt.LeftButton)
@@ -330,6 +412,10 @@ class TriblerGUITest(AbstractTriblerGUITest):
         self.screenshot(window, name="downloads_active")
         QTest.mouseClick(window.downloads_inactive_button, Qt.LeftButton)
         self.screenshot(window, name="downloads_inactive")
+        QTest.mouseClick(window.downloads_creditmining_button, Qt.LeftButton)
+        self.screenshot(window, name="downloads_creditmining")
+        QTest.mouseClick(window.downloads_channels_button, Qt.LeftButton)
+        self.screenshot(window, name="downloads_channels")
 
     def test_download_start_stop_remove_recheck(self):
         self.go_to_and_wait_for_downloads()
@@ -360,8 +446,9 @@ class TriblerGUITest(AbstractTriblerGUITest):
     def test_search(self):
         window.top_search_bar.setText("trib")
         QTest.keyClick(window.top_search_bar, Qt.Key_Enter)
-        self.wait_for_list_populated(window.search_results_list, num_items=20)
+        self.wait_for_list_populated(window.search_results_list)
         self.screenshot(window, name="search_results_all")
+        self.wait_for_something(window.search_results_list.model().total_items)
 
         QTest.mouseClick(window.search_results_channels_button, Qt.LeftButton)
         self.wait_for_list_populated(window.search_results_list)
@@ -370,96 +457,23 @@ class TriblerGUITest(AbstractTriblerGUITest):
         self.wait_for_list_populated(window.search_results_list)
         self.screenshot(window, name="search_results_torrents")
 
-    def test_channel_playlist(self):
-        QTest.mouseClick(window.left_menu_button_subscriptions, Qt.LeftButton)
-        self.wait_for_list_populated(window.subscribed_channels_list)
-        first_widget = window.subscribed_channels_list.itemWidget(window.subscribed_channels_list.item(0))
-        QTest.mouseClick(first_widget, Qt.LeftButton)
-        self.screenshot(window, name="channel_loading")
-        self.wait_for_list_populated(window.channel_torrents_list)
-        self.screenshot(window, name="channel")
-
-        first_widget = window.channel_torrents_list.itemWidget(window.channel_torrents_list.item(0))
-        QTest.mouseClick(first_widget, Qt.LeftButton)
-        self.screenshot(window, name="channel_playlist")
-
-    def test_start_download(self):
-        QTest.mouseClick(window.left_menu_button_subscriptions, Qt.LeftButton)
-        self.wait_for_list_populated(window.subscribed_channels_list)
-        first_widget = window.subscribed_channels_list.itemWidget(window.subscribed_channels_list.item(0))
-        QTest.mouseClick(first_widget, Qt.LeftButton)
-        self.wait_for_list_populated(window.channel_torrents_list)
-
-        torrent_widget = None
-        for ind in xrange(window.channel_torrents_list.count()):
-            cur_widget = window.channel_torrents_list.itemWidget(window.channel_torrents_list.item(ind))
-            if isinstance(cur_widget, ChannelTorrentListItem):
-                torrent_widget = cur_widget
-                break
-
-        QTest.mouseClick(torrent_widget.torrent_download_button, Qt.LeftButton)
-        self.screenshot(window, name="start_download_dialog")
-        QTest.mouseClick(window.dialog.dialog_widget.cancel_button, Qt.LeftButton)
-
-    def test_create_remove_playlist(self):
-        QTest.mouseClick(window.left_menu_button_my_channel, Qt.LeftButton)
-        self.wait_for_variable("edit_channel_page.channel_overview")
-        QTest.mouseClick(window.edit_channel_playlists_button, Qt.LeftButton)
-        self.wait_for_list_populated(window.edit_channel_playlists_list)
-        old_count = window.edit_channel_playlists_list.count()
-        QTest.mouseClick(window.edit_channel_create_playlist_button, Qt.LeftButton)
-        self.screenshot(window, "create_playlist")
-
-        # Create playlist
-        window.playlist_edit_name.setText("Unit test playlist")
-        window.playlist_edit_description.setText("Unit test playlist description")
-        QTest.mouseClick(window.playlist_edit_save_button, Qt.LeftButton)
-        self.wait_for_signal(window.edit_channel_page.playlists_loaded)
-        self.assertEqual(old_count + 1, window.edit_channel_playlists_list.count())
-
-        # Remove playlist
-        last_widget = window.edit_channel_playlists_list.itemWidget(window.edit_channel_playlists_list.item(old_count))
-        QTest.mouseClick(last_widget.remove_playlist_button, Qt.LeftButton)
-        self.screenshot(window, name="remove_playlist_dialog")
-        QTest.mouseClick(window.edit_channel_page.dialog.buttons[0], Qt.LeftButton)
-        self.wait_for_signal(window.edit_channel_page.playlists_loaded)
-        self.assertEqual(old_count, window.edit_channel_playlists_list.count())
-
-    def test_edit_playlist(self):
-        QTest.mouseClick(window.left_menu_button_my_channel, Qt.LeftButton)
-        self.wait_for_variable("edit_channel_page.channel_overview")
-        QTest.mouseClick(window.edit_channel_playlists_button, Qt.LeftButton)
-        self.wait_for_list_populated(window.edit_channel_playlists_list)
-
-        first_widget = window.edit_channel_playlists_list.itemWidget(window.edit_channel_playlists_list.item(0))
-        QTest.mouseClick(first_widget.edit_playlist_button, Qt.LeftButton)
-        self.screenshot(window, name="edit_playlist")
-
-        rand_name = "Random name %d" % randint(1, 1000)
-        rand_desc = "Random description %d" % randint(1, 1000)
-
-        window.playlist_edit_name.setText(rand_name)
-        window.playlist_edit_description.setText(rand_desc)
-        QTest.mouseClick(window.playlist_edit_save_button, Qt.LeftButton)
-        self.wait_for_signal(window.edit_channel_page.playlists_loaded)
-
-        first_widget = window.edit_channel_playlists_list.itemWidget(window.edit_channel_playlists_list.item(0))
-        self.assertEqual(first_widget.playlist_name.text(), rand_name)
-
+    @skipIf(sys.platform == "win32", "This test is unreliable on Windows")
     def test_add_download_url(self):
-        window.on_add_torrent_from_url()
         self.go_to_and_wait_for_downloads()
+        window.on_add_torrent_from_url()
         old_count = window.downloads_list.topLevelItemCount()
         self.screenshot(window, name="add_torrent_url_dialog")
         window.dialog.dialog_widget.dialog_input.setText("http://test.url/test.torrent")
         QTest.mouseClick(window.dialog.buttons[0], Qt.LeftButton)
+        QTest.qWait(200)
         self.screenshot(window, name="add_torrent_url_startdownload_dialog")
 
         # set the download directory to a writable path
         download_dir = os.path.join(os.path.expanduser("~"), "downloads")
         window.dialog.dialog_widget.destination_input.setCurrentText(download_dir)
 
-        self.wait_for_signal(window.dialog.received_metainfo)
+        self.wait_for_list_populated(window.dialog.dialog_widget.files_list_view)
+
         self.screenshot(window, name="add_torrent_url_startdownload_dialog_files")
         QTest.mouseClick(window.dialog.dialog_widget.download_button, Qt.LeftButton)
         self.wait_for_signal(window.downloads_page.received_downloads)
@@ -516,11 +530,6 @@ class TriblerGUITest(AbstractTriblerGUITest):
         QTimer.singleShot(1000, screenshot_dialog)
         dialog.exec_()
 
-    def test_discovered_page(self):
-        QTest.mouseClick(window.left_menu_button_discovered, Qt.LeftButton)
-        self.wait_for_list_populated(window.discovered_channels_list)
-        self.screenshot(window, name="discovered_page")
-
     def test_debug_pane(self):
         self.wait_for_variable("tribler_settings")
         QTest.mouseClick(window.settings_button, Qt.LeftButton)
@@ -587,7 +596,7 @@ class TriblerGUITest(AbstractTriblerGUITest):
         # logs from FakeTriblerApi
         fake_logs = ''.join(["Sample log [%d]\n" % i for i in xrange(10)]).strip()
 
-        window.debug_window.log_tab_widget.setCurrentIndex(0)   # Core tab
+        window.debug_window.log_tab_widget.setCurrentIndex(0)  # Core tab
         self.wait_for_qtext_edit_populated(window.debug_window.core_log_display_area)
         core_logs = window.debug_window.core_log_display_area.toPlainText().strip()
         self.assertEqual(core_logs, fake_logs, "Core logs found different than expected.")
@@ -617,37 +626,6 @@ class TriblerGUITest(AbstractTriblerGUITest):
 
         window.debug_window.close()
 
-    def test_create_torrent(self):
-        QTest.mouseClick(window.left_menu_button_my_channel, Qt.LeftButton)
-        self.wait_for_variable("edit_channel_page.channel_overview")
-        QTest.mouseClick(window.edit_channel_torrents_button, Qt.LeftButton)
-        self.wait_for_list_populated(window.edit_channel_torrents_list)
-        window.edit_channel_page.on_create_torrent_from_files()
-        self.screenshot(window, name="create_torrent_page")
-        QTest.mouseClick(window.manage_channel_create_torrent_back, Qt.LeftButton)
-
-    def test_manage_playlist(self):
-        QTest.mouseClick(window.left_menu_button_my_channel, Qt.LeftButton)
-        self.wait_for_variable("edit_channel_page.channel_overview")
-        QTest.mouseClick(window.edit_channel_playlists_button, Qt.LeftButton)
-        self.wait_for_list_populated(window.edit_channel_playlists_list)
-        first_widget = window.edit_channel_playlists_list.itemWidget(window.edit_channel_playlists_list.item(0))
-        QTest.mouseClick(first_widget, Qt.LeftButton)
-        QTest.mouseClick(window.edit_channel_playlist_manage_torrents_button, Qt.LeftButton)
-        self.wait_for_list_populated(window.playlist_manage_in_playlist_list)
-        self.screenshot(window, name="manage_playlist_before")
-
-        # Swap the first item of the lists around
-        window.playlist_manage_in_playlist_list.setCurrentRow(0)
-        QTest.mouseClick(window.playlist_manage_remove_from_playlist, Qt.LeftButton)
-
-        window.playlist_manage_in_channel_list.setCurrentRow(0)
-        QTest.mouseClick(window.playlist_manage_add_to_playlist, Qt.LeftButton)
-
-        self.screenshot(window, name="manage_playlist_after")
-
-        QTest.mouseClick(window.edit_channel_manage_playlist_save_button, Qt.LeftButton)
-
     def test_trust_page(self):
         QTest.mouseClick(window.token_balance_widget, Qt.LeftButton)
         self.wait_for_variable("trust_page.blocks")
@@ -756,6 +734,3 @@ class TriblerGUITest(AbstractTriblerGUITest):
         QTest.keyClick(window.market_page.dialog.dialog_widget.order_price_input, 't')
         QTest.mouseClick(window.market_page.dialog.dialog_widget.create_button, Qt.LeftButton)
         self.screenshot(window, name="market_create_order_dialog_error")
-
-if __name__ == "__main__":
-    unittest.main()
