@@ -40,7 +40,7 @@ class Peer(object):
         :param timestamp: a received timestamp
         """
         self._lamport_timestamp = max(self._lamport_timestamp, timestamp)
-        self.last_response = time() # This is in seconds since the epoch
+        self.last_response = time()  # This is in seconds since the epoch
 
     def get_lamport_timestamp(self):
         return self._lamport_timestamp
@@ -55,4 +55,4 @@ class Peer(object):
         return (self.public_key.key_to_bin() == other.public_key.key_to_bin())
 
     def __str__(self):
-        return 'Peer<%s:%d, %s>' % (self.address[0], self.address[1], b64encode(self.mid))
+        return 'Peer<%s:%d, %s>' % (self.address[0], self.address[1], b64encode(self.mid).decode('utf-8'))
