@@ -4,13 +4,13 @@ import datetime
 import logging
 import os
 import socket
-from time import localtime, strftime
+from time import localtime, strftime, time
 
 from PyQt5 import QtGui, uic
 from PyQt5.QtCore import QTimer, Qt, pyqtSignal
-from PyQt5.QtGui import QTextCursor
-from PyQt5.QtWidgets import QDesktopWidget, QFileDialog, QHeaderView, QMainWindow, QMessageBox, QSizePolicy, \
-    QTreeWidgetItem
+from PyQt5.QtGui import QBrush, QColor, QTextCursor
+from PyQt5.QtWidgets import (QDesktopWidget, QFileDialog, QHeaderView, QMainWindow, QMessageBox, QSizePolicy,
+                             QTreeWidgetItem)
 
 import matplotlib
 matplotlib.use('Qt5Agg')
@@ -18,7 +18,11 @@ from matplotlib.backends.backend_qt5agg import FigureCanvas
 from matplotlib.dates import DateFormatter
 from matplotlib.figure import Figure
 
-from meliae import scanner
+try:
+    from meliae import scanner
+except ImportError:
+    scanner = None
+
 
 import psutil
 
@@ -243,6 +247,8 @@ class DebugWindow(QMainWindow):
             self.run_with_timer(self.load_tunnel_relays_tab)
         elif index == 2:
             self.run_with_timer(self.load_tunnel_exits_tab)
+        elif index == 3:
+            self.run_with_timer(self.load_tunnel_swarms_tab)
 
     def system_tab_changed(self, index):
         if index == 0:
@@ -275,13 +281,9 @@ class DebugWindow(QMainWindow):
         self.window().general_tree_widget.clear()
         self.create_and_add_widget_item("Tribler version", self.tribler_version, self.window().general_tree_widget)
         self.create_and_add_widget_item("Number of channels", data["num_channels"], self.window().general_tree_widget)
-        self.create_and_add_widget_item("Database size", format_size(data["database_size"]),
+        self.create_and_add_widget_item("Database size", format_size(data["db_size"]),
                                         self.window().general_tree_widget)
-        self.create_and_add_widget_item("Number of collected torrents", data["torrents"]["num_collected"],
-                                        self.window().general_tree_widget)
-        self.create_and_add_widget_item("Number of torrent files", data["torrents"]["num_files"],
-                                        self.window().general_tree_widget)
-        self.create_and_add_widget_item("Total size of torrent files", format_size(data["torrents"]["total_size"]),
+        self.create_and_add_widget_item("Number of known torrents", data["num_torrents"],
                                         self.window().general_tree_widget)
         self.create_and_add_widget_item("", "", self.window().general_tree_widget)
 
@@ -298,7 +300,7 @@ class DebugWindow(QMainWindow):
         for endpoint, method, data, timestamp, status_code in sorted(tribler_performed_requests,
                                                                      key=lambda x: x[3]):
             item = QTreeWidgetItem(self.window().requests_tree_widget)
-            item.setText(0, "%s %s %s" % (method, endpoint, data))
+            item.setText(0, "%s %s %s" % (method, repr(endpoint), repr(data)))
             item.setText(1, ("%d" % status_code) if status_code else "unknown")
             item.setText(2, "%s" % strftime("%H:%M:%S", localtime(timestamp)))
             self.window().requests_tree_widget.addTopLevelItem(item)
@@ -311,7 +313,7 @@ class DebugWindow(QMainWindow):
         if not data:
             return
         self.window().trustchain_tree_widget.clear()
-        for key, value in data["statistics"].iteritems():
+        for key, value in data["statistics"].items():
             self.create_and_add_widget_item(key, value, self.window().trustchain_tree_widget)
 
     def load_ipv8_general_tab(self):
@@ -322,27 +324,40 @@ class DebugWindow(QMainWindow):
         if not data:
             return
         self.window().ipv8_general_tree_widget.clear()
-        for key, value in data["ipv8_statistics"].iteritems():
-            if key == 'total_up' or key == 'total_down':
+        for key, value in data["ipv8_statistics"].items():
+            if key in ('total_up', 'total_down'):
                 value = "%.2f MB" % (value / (1024.0 * 1024.0))
-            if key == 'session_uptime':
+            elif key == 'session_uptime':
                 value = "%s" % str(datetime.timedelta(seconds=int(value)))
             self.create_and_add_widget_item(key, value, self.window().ipv8_general_tree_widget)
 
     def load_ipv8_communities_tab(self):
         self.request_mgr = TriblerRequestManager()
-        self.request_mgr.perform_request("statistics/communities", self.on_ipv8_community_stats)
+        self.request_mgr.perform_request("ipv8/overlays", self.on_ipv8_community_stats)
+
+    def _colored_peer_count(self, peer_count, overlay_count, master_peer):
+        is_discovery = (master_peer == "3081a7301006072a8648ce3d020106052b81040027038192000403b3ab059ced9b20646ab5e01"
+                                       "762b3595c5e8855227ae1e424cff38a1e4edee73734ff2e2e829eb4f39bab20d7578284fcba72"
+                                       "51acd74e7daf96f21d01ea17077faf4d27a655837d072baeb671287a88554e1191d8904b0dc57"
+                                       "2d09ff95f10ff092c8a5e2a01cd500624376aec875a6e3028aab784cfaf0bac6527245db8d939"
+                                       "00d904ac2a922a02716ccef5a22f7968")
+        limits = [20, overlay_count * 30 + 1] if is_discovery else [20, 31]
+        color = 0xF4D03F if peer_count < limits[0] else (0x56F129 if peer_count < limits[1] else 0xF12929)
+        return QBrush(QColor(color))
 
     def on_ipv8_community_stats(self, data):
         if not data:
             return
         self.window().communities_tree_widget.clear()
-        for overlay in data["ipv8_overlay_statistics"]:
+        for overlay in data["overlays"]:
             item = QTreeWidgetItem(self.window().communities_tree_widget)
             item.setText(0, overlay["overlay_name"])
             item.setText(1, overlay["master_peer"][-12:])
             item.setText(2, overlay["my_peer"][-12:])
-            item.setText(3, "%s" % len(overlay["peers"]))
+            peer_count = len(overlay["peers"])
+            item.setText(3, "%s" % peer_count)
+            item.setForeground(3, self._colored_peer_count(peer_count, len(data["overlays"]),
+                                                           overlay["master_peer"]))
 
             if "statistics" in overlay and overlay["statistics"]:
                 statistics = overlay["statistics"]
@@ -379,14 +394,14 @@ class DebugWindow(QMainWindow):
         for overlay in data["statistics"]:
             self.window().ipv8_communities_details_widget.setColumnWidth(0, 250)
 
-            for key, stats in overlay.iteritems():
+            for key, stats in overlay.items():
                 header_item = QTreeWidgetItem(self.window().ipv8_communities_details_widget)
                 header_item.setFirstColumnSpanned(True)
                 header_item.setBackground(0, QtGui.QColor('#CCCCCC'))
                 header_item.setText(0, key)
                 self.window().ipv8_communities_details_widget.addTopLevelItem(header_item)
 
-                for request_id, stat in stats.iteritems():
+                for request_id, stat in stats.items():
                     stat_item = QTreeWidgetItem(self.window().ipv8_communities_details_widget)
                     stat_item.setText(0, request_id)
                     stat_item.setText(1, "%.3f" % (stat["bytes_up"] / (1024.0 * 1024.0)))
@@ -400,7 +415,12 @@ class DebugWindow(QMainWindow):
         for item in items:
             widget_item = QTreeWidgetItem(tree)
             for index, key in enumerate(keys):
-                value = format_size(item[key]) if key in ["bytes_up", "bytes_down"] else str(item[key])
+                if key in ["bytes_up", "bytes_down"]:
+                    value = format_size(item[key])
+                elif key in ["creation_time", "last_lookup"]:
+                    value = str(datetime.timedelta(seconds=int(time() - item[key]))) if item[key] > 0 else '-'
+                else:
+                    value = str(item[key])
                 widget_item.setText(index, value)
             tree.addTopLevelItem(widget_item)
 
@@ -411,8 +431,8 @@ class DebugWindow(QMainWindow):
     def on_tunnel_circuits(self, data):
         if data:
             self.add_items_to_tree(self.window().circuits_tree_widget, data.get("circuits"),
-                                   ["circuit_id", "goal_hops", "actual_hops",
-                                    "type", "state", "bytes_up", "bytes_down"])
+                                   ["circuit_id", "goal_hops", "actual_hops", "unverified_hop",
+                                    "type", "state", "bytes_up", "bytes_down", "creation_time"])
 
     def load_tunnel_relays_tab(self):
         self.request_mgr = TriblerRequestManager()
@@ -421,7 +441,8 @@ class DebugWindow(QMainWindow):
     def on_tunnel_relays(self, data):
         if data:
             self.add_items_to_tree(self.window().relays_tree_widget, data["relays"],
-                                   ["circuit_from", "circuit_to", "is_rendezvous", "bytes_up", "bytes_down"])
+                                   ["circuit_from", "circuit_to", "is_rendezvous",
+                                    "bytes_up", "bytes_down", "creation_time"])
 
     def load_tunnel_exits_tab(self):
         self.request_mgr = TriblerRequestManager()
@@ -430,7 +451,17 @@ class DebugWindow(QMainWindow):
     def on_tunnel_exits(self, data):
         if data:
             self.add_items_to_tree(self.window().exits_tree_widget, data["exits"],
-                                   ["circuit_from", "enabled", "bytes_up", "bytes_down"])
+                                   ["circuit_from", "enabled", "bytes_up", "bytes_down", "creation_time"])
+
+    def load_tunnel_swarms_tab(self):
+        self.request_mgr = TriblerRequestManager()
+        self.request_mgr.perform_request("ipv8/tunnel/swarms", self.on_tunnel_swarms)
+
+    def on_tunnel_swarms(self, data):
+        if data:
+            self.add_items_to_tree(self.window().swarms_tree_widget, data.get("swarms"),
+                                   ["info_hash", "num_seeders", "num_connections", "num_connections_incomplete",
+                                    "seeding", "last_lookup", "bytes_up", "bytes_down"])
 
     def load_dht_tab(self):
         self.request_mgr = TriblerRequestManager()
@@ -440,7 +471,7 @@ class DebugWindow(QMainWindow):
         if not data:
             return
         self.window().dht_tree_widget.clear()
-        for key, value in data["statistics"].iteritems():
+        for key, value in data["statistics"].items():
             self.create_and_add_widget_item(key, value, self.window().dht_tree_widget)
 
     def on_event_clicked(self, item):
@@ -649,8 +680,12 @@ class DebugWindow(QMainWindow):
                 self.request_mgr = TriblerRequestManager()
                 self.request_mgr.download_file("debug/memory/dump",
                                                lambda data: self.on_memory_dump_data_available(filename, data))
-            else:
+            elif scanner:
                 scanner.dump_all_objects(os.path.join(self.export_dir, filename))
+            else:
+                ConfirmationDialog.show_error(self.window(),
+                                              "Error when performing a memory dump",
+                                              "meliae memory dumper is not compatible with Python 3")
 
     def on_memory_dump_data_available(self, filename, data):
         if not data:
@@ -737,7 +772,7 @@ class DebugWindow(QMainWindow):
     def on_libtorrent_settings_received(self, data, export=False):
         if not data:
             return
-        for key, value in data["settings"].iteritems():
+        for key, value in data["settings"].items():
             item = QTreeWidgetItem(self.window().libtorrent_settings_tree_widget)
             item.setText(0, key)
             item.setText(1, str(value))
@@ -754,7 +789,7 @@ class DebugWindow(QMainWindow):
     def on_libtorrent_session_received(self, data, export=False):
         if not data:
             return
-        for key, value in data["session"].iteritems():
+        for key, value in data["session"].items():
             item = QTreeWidgetItem(self.window().libtorrent_session_tree_widget)
             item.setText(0, key)
             item.setText(1, str(value))
