@@ -3,34 +3,40 @@ A wrapper around a libtorrent download.
 
 Author(s): Arno Bakker, Egbert Bouman
 """
+from __future__ import absolute_import
+
 import base64
 import logging
 import os
-import random
+import shutil
 import sys
 import time
 from binascii import hexlify
-from traceback import print_exc
-from twisted.internet import reactor
-from twisted.internet.defer import Deferred, CancelledError, succeed
-from twisted.internet.task import LoopingCall
-from twisted.python.failure import Failure
+from threading import RLock
+
+from ipv8.taskmanager import TaskManager
 
 import libtorrent as lt
 
-from Tribler.Core import NoDispersyRLock
-from Tribler.Core.DownloadConfig import DownloadStartupConfig, DownloadConfigInterface, get_default_dest_dir
+import six
+from six.moves import xrange
+
+from twisted.internet import reactor
+from twisted.internet.defer import CancelledError, Deferred, succeed
+from twisted.internet.task import LoopingCall
+from twisted.python.failure import Failure
+
+from Tribler.Core.DownloadConfig import DownloadConfigInterface, DownloadStartupConfig, get_default_dest_dir
 from Tribler.Core.DownloadState import DownloadState
 from Tribler.Core.Libtorrent import checkHandleAndSynchronize
-from Tribler.Core.TorrentDef import TorrentDefNoMetainfo, TorrentDef
+from Tribler.Core.TorrentDef import TorrentDef, TorrentDefNoMetainfo
 from Tribler.Core.Utilities import maketorrent
 from Tribler.Core.Utilities.torrent_utils import get_info_from_handle
+from Tribler.Core.Utilities.unicode import ensure_unicode
 from Tribler.Core.exceptions import SaveResumeDataError
 from Tribler.Core.osutils import fix_filebasename
-from Tribler.Core.simpledefs import DLSTATUS_SEEDING, DLSTATUS_STOPPED, DLMODE_VOD, DLMODE_NORMAL, \
-                                    PERSISTENTSTATE_CURRENTVERSION, dlstatus_strings
-from Tribler.pyipv8.ipv8.taskmanager import TaskManager
-
+from Tribler.Core.simpledefs import (
+    DLMODE_NORMAL, DLMODE_VOD, DLSTATUS_SEEDING, DLSTATUS_STOPPED, PERSISTENTSTATE_CURRENTVERSION, dlstatus_strings)
 
 if sys.platform == "win32":
     try:
@@ -48,7 +54,7 @@ class VODFile(object):
         self._download = d
 
         pieces = self._download.tdef.get_pieces()
-        self.pieces = [pieces[x:x + 20]for x in xrange(0, len(pieces), 20)]
+        self.pieces = [pieces[x:x + 20] for x in xrange(0, len(pieces), 20)]
         self.piecesize = self._download.tdef.get_piece_length()
 
         self.startpiece = get_info_from_handle(self._download.handle).map_file(
@@ -61,7 +67,9 @@ class VODFile(object):
 
         self._logger.debug('VODFile: get bytes %s - %s', oldpos, oldpos + args[0])
 
-        while not self._file.closed and self._download.get_byte_progress([(self._download.get_vod_fileindex(), oldpos, oldpos + args[0])]) < 1 and self._download.vod_seekpos is not None:
+        while not self._file.closed and self._download.get_byte_progress([
+            (self._download.get_vod_fileindex(), oldpos, oldpos + args[0])]) < 1 \
+                and self._download.vod_seekpos is not None:
             time.sleep(1)
 
         if self._file.closed:
@@ -91,7 +99,7 @@ class VODFile(object):
 
         self._logger.debug('VODFile: seek, get pieces %s', self._download.handle.piece_priorities())
         self._logger.debug('VODFile: seek, got pieces %s', [
-                           int(piece) for piece in self._download.handle.status().pieces])
+            int(piece) for piece in self._download.handle.status().pieces])
 
     def close(self, *args):
         self._file.close(*args)
@@ -102,7 +110,6 @@ class VODFile(object):
 
 
 class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
-
     """ Download subclass that represents a libtorrent download."""
 
     def __init__(self, session, tdef):
@@ -110,12 +117,21 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
         self._logger = logging.getLogger(self.__class__.__name__)
 
-        self.dllock = NoDispersyRLock()
+        self.dllock = RLock()
         self.session = session
         self.tdef = tdef
         self.handle = None
         self.vod_index = None
         self.orig_files = None
+        self.finished_deferred = Deferred()
+        self.finished_deferred_already_called = False
+        self.is_bootstrap_download = False
+
+        if session:
+            self.state_dir = self.session.config.get_state_dir()
+
+        # With hidden True download will not be in GET/downloads set, as a result will not be shown in GUI
+        self.hidden = False
 
         # To be able to return the progress of a stopped torrent, how far it got.
         self.progressbeforestop = 0.0
@@ -191,7 +207,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         self.deferreds_handle.append(deferred)
         return deferred
 
-    def setup(self, dcfg=None, pstate=None, wrapperDelay=0, share_mode=False, checkpoint_disabled=False):
+    def setup(self, dcfg=None, pstate=None, wrapperDelay=0, share_mode=False, checkpoint_disabled=False, hidden=False):
         """
         Create a Download object. Used internally by Session.
         @param dcfg DownloadStartupConfig or None (in which case
@@ -202,16 +218,15 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         """
         # Called by any thread, assume sessionlock is held
         self.set_checkpoint_disabled(checkpoint_disabled)
-
+        self.hidden = hidden
         try:
             # The deferred to be returned
             deferred = Deferred()
             with self.dllock:
                 # Copy dlconfig, from default if not specified
-                if dcfg is None:
-                    cdcfg = DownloadStartupConfig()
-                else:
-                    cdcfg = dcfg
+                cdcfg = dcfg or DownloadStartupConfig(state_dir=self.session.config.get_state_dir())
+
+                self.is_bootstrap_download = cdcfg.is_bootstrap_download
                 self.dlconfig = cdcfg.dlconfig.copy()
                 self.dlconfig.lock = self.dllock
                 self.dlconfig.set_callback(self.dlconfig_changed_callback)
@@ -229,13 +244,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                         checkpoint_disabled=checkpoint_disabled)
                     create_engine_wrapper_deferred.chainDeferred(deferred)
 
-                def schedule_create_engine_call(_):
-                    self.register_task("schedule_create_engine",
-                                       reactor.callLater(wrapperDelay, schedule_create_engine))
-
-                # Add a lambda callback that ignored the parameter of the callback which schedules
-                # a task using the taskamanger with wrapperDelay as delay.
-                self.can_create_engine_wrapper().addCallback(schedule_create_engine_call)
+                self.register_task("schedule_create_engine", reactor.callLater(wrapperDelay, schedule_create_engine))
 
             self.pstate_for_restart = pstate
             self.checkpoint()
@@ -245,52 +254,23 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         except Exception as e:
             self.error = e
 
-    def can_create_engine_wrapper(self):
-        """
-        Periodically checks whether the engine wrapper can be created.
-        Notifies when it's ready by calling the callback of the deferred being returned.
-        :return: A deferred that will be called when you can create the engine wrapper.
-        """
-        can_create_deferred = Deferred()
-        def do_check():
-            with self.dllock:
-                if not self.cew_scheduled:
-                    self.ltmgr = self.session.lm.ltmgr
-                    dht_ok = not isinstance(self.tdef, TorrentDefNoMetainfo) or self.ltmgr.is_dht_ready()
-                    tunnel_community = self.ltmgr.tribler_session.lm.tunnel_community
-                    tunnels_ready = tunnel_community.tunnels_ready(self.get_hops()) if tunnel_community else 1
-
-                    if not self.ltmgr or not dht_ok or tunnels_ready < 1:
-                        self._logger.info(u"LTMGR/DHT/session not ready, rescheduling create_engine_wrapper")
-
-                        if tunnel_community and tunnels_ready < 1:
-                            tunnel_community.build_tunnels(self.get_hops())
-
-                        # Schedule this function call to be called again in 5 seconds
-                        self.register_task("check_create_wrapper", reactor.callLater(5, do_check))
-                    else:
-                        can_create_deferred.callback(True)
-                else:
-                    # Schedule this function call to be called again in 5 seconds
-                    self.register_task("check_create_wrapper", reactor.callLater(5, do_check))
-
-        do_check()
-        return can_create_deferred
-
-    def network_create_engine_wrapper(self, pstate, checkpoint_disabled=False, share_mode=False):
+    def network_create_engine_wrapper(self, pstate, checkpoint_disabled=False, share_mode=False, upload_mode=False):
+        self.ltmgr = self.session.lm.ltmgr
         with self.dllock:
             self._logger.debug("LibtorrentDownloadImpl: network_create_engine_wrapper()")
 
             atp = {}
             atp["save_path"] = os.path.normpath(os.path.join(get_default_dest_dir(), self.get_dest_dir()))
             atp["storage_mode"] = lt.storage_mode_t.storage_mode_sparse
-            atp["paused"] = True
-            atp["auto_managed"] = False
-            atp["duplicate_is_error"] = False
             atp["hops"] = self.get_hops()
+            atp["flags"] = lt.add_torrent_params_flags_t.flag_paused | \
+                           lt.add_torrent_params_flags_t.flag_duplicate_is_error | \
+                           lt.add_torrent_params_flags_t.flag_update_subscribe
 
             if share_mode:
-                atp["flags"] = lt.add_torrent_params_flags_t.flag_share_mode
+                atp["flags"] = atp["flags"] | lt.add_torrent_params_flags_t.flag_share_mode
+            if upload_mode:
+                atp["flags"] = atp["flags"] | lt.add_torrent_params_flags_t.flag_upload_mode
 
             self.set_checkpoint_disabled(checkpoint_disabled)
 
@@ -316,8 +296,11 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                         self.orig_files[i] = filename_new
 
                 atp["ti"] = torrentinfo
-                has_resume_data = resume_data and isinstance(resume_data, dict)
-                if has_resume_data:
+                if resume_data and isinstance(resume_data, dict):
+                    # Rewrite save_path as a global path, if it is given as a relative path
+                    if "save_path" in resume_data and not os.path.isabs(resume_data["save_path"]):
+                        resume_data["save_path"] = six.text_type(
+                            os.path.join(self.state_dir, ensure_unicode(resume_data["save_path"], 'utf-8')))
                     atp["resume_data"] = lt.bencode(resume_data)
             else:
                 atp["url"] = self.tdef.get_url() or "magnet:?xt=urn:btih:" + hexlify(self.tdef.get_infohash())
@@ -326,7 +309,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         def on_torrent_added(handle):
             self.handle = handle
 
-            if self.handle.is_valid():
+            if self.handle and self.handle.is_valid():
                 self.set_selected_files()
 
                 user_stopped = pstate.get('download_defaults', 'user_stopped') if pstate else False
@@ -342,6 +325,11 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
                 # Limit the amount of connections if we have specified that
                 self.handle.set_max_connections(self.session.config.get_libtorrent_max_conn_download())
+
+                # Set limit on download for a bootstrap file
+                if self.is_bootstrap_download:
+                    self.handle.set_download_limit(self.session.config.get_bootstrap_max_download_rate())
+
                 return self
 
         def on_torrent_failed(failure):
@@ -389,8 +377,9 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
     @checkHandleAndSynchronize(0)
     def get_vod_filesize(self):
         fileindex = self.get_vod_fileindex()
-        if fileindex >= 0:
-            file_entry = get_info_from_handle(self.handle).file_at(fileindex)
+        torrent_info = get_info_from_handle(self.handle)
+        if fileindex >= 0 and torrent_info:
+            file_entry = torrent_info.file_at(fileindex)
             return file_entry.size
         return 0
 
@@ -425,24 +414,28 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
         encoded_str = ""
         for i in range(0, len(bitstr), 8):
-            encoded_str += chr(int(bitstr[i:i+8].ljust(8, '0'), 2))
+            encoded_str += chr(int(bitstr[i:i + 8].ljust(8, '0'), 2))
         return base64.b64encode(encoded_str)
 
     @checkHandleAndSynchronize(0.0)
     def get_byte_progress(self, byteranges, consecutive=False):
         pieces = []
+        torrent_info = get_info_from_handle(self.handle)
+        if not torrent_info:
+            self._logger.info("LibtorrentDownloadImpl: could not get info from download handle")
+
         for fileindex, bytes_begin, bytes_end in byteranges:
-            if fileindex >= 0:
+            if fileindex >= 0 and torrent_info:
                 # Ensure the we remain within the file's boundaries
-                file_entry = get_info_from_handle(self.handle).file_at(fileindex)
+                file_entry = torrent_info.file_at(fileindex)
                 bytes_begin = min(
                     file_entry.size, bytes_begin) if bytes_begin >= 0 else file_entry.size + (bytes_begin + 1)
                 bytes_end = min(file_entry.size, bytes_end) if bytes_end >= 0 else file_entry.size + (bytes_end + 1)
 
-                startpiece = get_info_from_handle(self.handle).map_file(fileindex, bytes_begin, 0).piece
-                endpiece = get_info_from_handle(self.handle).map_file(fileindex, bytes_end, 0).piece + 1
+                startpiece = torrent_info.map_file(fileindex, bytes_begin, 0).piece
+                endpiece = torrent_info.map_file(fileindex, bytes_end, 0).piece + 1
                 startpiece = max(startpiece, 0)
-                endpiece = min(endpiece, get_info_from_handle(self.handle).num_pieces())
+                endpiece = min(endpiece, torrent_info.num_pieces())
 
                 pieces += range(startpiece, endpiece)
             else:
@@ -463,7 +456,8 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     do_prio = True
             else:
                 self._logger.info(
-                    "LibtorrentDownloadImpl: could not set priority for non-existing piece %d / %d", piece, len(piecepriorities))
+                    "LibtorrentDownloadImpl: could not set priority for non-existing piece %d / %d", piece,
+                    len(piecepriorities))
         if do_prio:
             self.handle.prioritize_pieces(piecepriorities)
         else:
@@ -472,18 +466,22 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
     @checkHandleAndSynchronize()
     def set_byte_priority(self, byteranges, priority):
         pieces = []
+        torrent_info = get_info_from_handle(self.handle)
+        if not torrent_info:
+            self._logger.info("LibtorrentDownloadImpl: could not get info from download handle")
+
         for fileindex, bytes_begin, bytes_end in byteranges:
-            if fileindex >= 0:
+            if fileindex >= 0 and torrent_info:
                 # Ensure the we remain within the file's boundaries
-                file_entry = get_info_from_handle(self.handle).file_at(fileindex)
+                file_entry = torrent_info.file_at(fileindex)
                 bytes_begin = min(
                     file_entry.size, bytes_begin) if bytes_begin >= 0 else file_entry.size + (bytes_begin + 1)
                 bytes_end = min(file_entry.size, bytes_end) if bytes_end >= 0 else file_entry.size + (bytes_end + 1)
 
-                startpiece = get_info_from_handle(self.handle).map_file(fileindex, bytes_begin, 0).piece
-                endpiece = get_info_from_handle(self.handle).map_file(fileindex, bytes_end, 0).piece + 1
+                startpiece = torrent_info.map_file(fileindex, bytes_begin, 0).piece
+                endpiece = torrent_info.map_file(fileindex, bytes_end, 0).piece + 1
                 startpiece = max(startpiece, 0)
-                endpiece = min(endpiece, get_info_from_handle(self.handle).num_pieces())
+                endpiece = min(endpiece, torrent_info.num_pieces())
 
                 pieces += range(startpiece, endpiece)
             else:
@@ -500,7 +498,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
         alert_types = ('tracker_reply_alert', 'tracker_error_alert', 'tracker_warning_alert', 'metadata_received_alert',
                        'file_renamed_alert', 'performance_alert', 'torrent_checked_alert', 'torrent_finished_alert',
-                       'save_resume_data_alert', 'save_resume_data_failed_alert', 'state_update_alert')
+                       'save_resume_data_alert', 'save_resume_data_failed_alert')
 
         if alert_type in alert_types:
             getattr(self, 'on_' + alert_type)(alert)
@@ -516,6 +514,14 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         resume_data = alert.resume_data
 
         self.pstate_for_restart = self.get_persistent_download_config()
+
+        # Make save_path relative if the torrent is saved in the Tribler state directory
+        if self.state_dir and 'save_path' in resume_data and os.path.abspath(resume_data['save_path']):
+            if self.state_dir == os.path.commonprefix([ensure_unicode(resume_data['save_path'], 'utf-8'),
+                                                       self.state_dir]):
+                resume_data['save_path'] = six.text_type(
+                    os.path.relpath(ensure_unicode(resume_data['save_path'], 'utf-8'), self.state_dir))
+
         self.pstate_for_restart.set('state', 'engineresumedata', resume_data)
         self._logger.debug("%s get resume data %s", hexlify(resume_data['info-hash']), resume_data)
 
@@ -595,16 +601,12 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         self.set_filepieceranges()
         self.set_selected_files()
 
-        if self.session.lm.rtorrent_handler:
-            self.session.lm.rtorrent_handler.save_torrent(self.tdef)
-        elif self.session.lm.torrent_db:
-            self.session.lm.torrent_db.addExternalTorrent(self.tdef, extra_info={'status': 'good'})
-
         self.checkpoint()
 
     def on_file_renamed_alert(self, alert):
-        if os.path.exists(self.unwanteddir_abs) and not os.listdir(self.unwanteddir_abs) and all(self.handle.file_priorities()):
-            os.rmdir(self.unwanteddir_abs)
+        unwanteddir_abs = os.path.join(ensure_unicode(self.get_save_path(), 'utf-8'), self.unwanted_directory_name)
+        if os.path.exists(unwanteddir_abs) and all(self.handle.file_priorities()):
+            shutil.rmtree(unwanteddir_abs, ignore_errors=True)
 
     def on_performance_alert(self, alert):
         if self.get_anon_mode() or self.ltmgr.ltsessions is None:
@@ -619,7 +621,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     "LibtorrentDownloadImpl: setting send_buffer_watermark to %s",
                     2 * settings['send_buffer_watermark'])
                 settings['send_buffer_watermark'] *= 2
-                self.ltmgr.get_session().set_settings(settings)
+                self.ltmgr.set_session_settings(self.ltmgr.get_session(), settings)
         # When the write cache is too small, double the buffer size to a maximum
         # of 64MiB. Again, this is the same mechanism as Deluge uses.
         elif alert.message().endswith("max outstanding disk writes reached"):
@@ -629,7 +631,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     "LibtorrentDownloadImpl: setting max_queued_disk_bytes to %s",
                     2 * settings['max_queued_disk_bytes'])
                 settings['max_queued_disk_bytes'] *= 2
-                self.ltmgr.get_session().set_settings(settings)
+                self.ltmgr.set_session_settings(self.ltmgr.get_session(), settings)
 
     def on_torrent_checked_alert(self, alert):
         if self.pause_after_next_hashcheck:
@@ -642,6 +644,13 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
     @checkHandleAndSynchronize()
     def on_torrent_finished_alert(self, alert):
         self.update_lt_status(self.handle.status())
+        if self.finished_deferred_already_called:
+            self._logger.warning("LibtorrentDownloadImpl: tried to repeat the call to finished_callback %s",
+                                 self.tdef.get_name())
+        else:
+            self.finished_deferred.callback(self)
+            self.finished_deferred_already_called = True
+
         progress = self.get_state().get_progress()
         if self.get_mode() == DLMODE_VOD:
             if progress == 1.0:
@@ -657,6 +666,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                         return
                     if self.get_state().get_progress() == 1.0:
                         self.set_byte_priority([(self.get_vod_fileindex(), 0, -1)], 1)
+
                 self.register_anonymous_task("reset_priorities", reactor.callLater(5, reset_priorities))
 
             if self.endbuffsize:
@@ -682,9 +692,26 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         self.correctedinfoname = fix_filebasename(self.tdef.get_name_as_unicode())
 
         # Allow correctedinfoname to be overwritten for multifile torrents only
-        if self.get_corrected_filename() and self.get_corrected_filename() != '' and 'files' in self.tdef.get_metainfo()['info']:
+        if self.get_corrected_filename() and self.get_corrected_filename() != '' and 'files' in \
+                self.tdef.get_metainfo()['info']:
             self.correctedinfoname = self.get_corrected_filename()
 
+    @property
+    def swarmname(self):
+        """
+        Return the swarm name of the torrent.
+        """
+        is_multifile = len(self.orig_files) > 1
+        commonprefix = os.path.commonprefix(self.orig_files) if is_multifile else u''
+        return commonprefix.partition(os.path.sep)[0]
+
+    @property
+    def unwanted_directory_name(self):
+        """
+        Return the name of the directory containing the unwanted files (files with a priority of 0).
+        """
+        return os.path.join(self.swarmname, u'.unwanted')
+
     @checkHandleAndSynchronize()
     def set_selected_files(self, selected_files=None):
         if not isinstance(self.tdef, TorrentDefNoMetainfo):
@@ -694,24 +721,25 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
             else:
                 DownloadConfigInterface.set_selected_files(self, selected_files)
 
-            is_multifile = len(self.orig_files) > 1
-            commonprefix = os.path.commonprefix(self.orig_files) if is_multifile else u''
-            swarmname = commonprefix.partition(os.path.sep)[0]
-            unwanteddir = os.path.join(swarmname, u'.unwanted')
-            unwanteddir_abs = os.path.join(self.get_save_path().decode('utf-8'), unwanteddir)
+            unwanteddir_abs = os.path.join(self.get_save_path().decode('utf-8'), self.unwanted_directory_name)
+            torrent_info = get_info_from_handle(self.handle)
+            if not torrent_info or not hasattr(torrent_info, 'files'):
+                self._logger.error("File info not available for torrent [%s]", self.correctedinfoname)
+                return
 
             filepriorities = []
-            torrent_storage = get_info_from_handle(self.handle).files()
+            torrent_storage = torrent_info.files()
 
             for index, orig_path in enumerate(self.orig_files):
-                filename = orig_path[len(swarmname) + 1:] if swarmname else orig_path
+                filename = orig_path[len(self.swarmname) + 1:] if self.swarmname else orig_path
 
                 if filename in selected_files or not selected_files:
                     filepriorities.append(1)
                     new_path = orig_path
                 else:
                     filepriorities.append(0)
-                    new_path = os.path.join(unwanteddir, '%s%d' % (hexlify(self.tdef.get_infohash()), index))
+                    new_path = os.path.join(self.unwanted_directory_name,
+                                            '%s%d' % (hexlify(self.tdef.get_infohash()), index))
 
                 # as from libtorrent 1.0, files returning file_storage (lazy-iterable)
                 if hasattr(lt, 'file_storage') and isinstance(torrent_storage, lt.file_storage):
@@ -720,7 +748,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     cur_path = torrent_storage[index].path.decode('utf-8')
 
                 if cur_path != new_path:
-                    if not os.path.exists(unwanteddir_abs) and unwanteddir in new_path:
+                    if not os.path.exists(unwanteddir_abs) and self.unwanted_directory_name in new_path:
                         try:
                             os.makedirs(unwanteddir_abs)
                             if sys.platform == "win32":
@@ -742,8 +770,6 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
             if not self.get_share_mode():
                 self.handle.prioritize_files(filepriorities)
 
-            self.unwanteddir_abs = unwanteddir_abs
-
     @checkHandleAndSynchronize(False)
     def move_storage(self, new_dir):
         if not isinstance(self.tdef, TorrentDefNoMetainfo):
@@ -758,9 +784,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
             # self.handle.status().save_path to query the save path of a torrent. However, this attribute
             # is only included in libtorrent 1.0.9+
             status = self.handle.status()
-            if hasattr(status, 'save_path'):
-                return status.save_path
-            return self.handle.save_path()
+            return status.save_path if hasattr(status, 'save_path') else self.handle.save_path()
 
     @checkHandleAndSynchronize()
     def force_recheck(self):
@@ -811,8 +835,9 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     [(self.get_vod_fileindex(), self.vod_seekpos, self.vod_seekpos + self.prebuffsize),
                      (self.get_vod_fileindex(), -self.endbuffsize - 1, -1)], consecutive=consecutive)
             else:
-                return self.get_byte_progress([(self.get_vod_fileindex(), self.vod_seekpos, self.vod_seekpos + self.prebuffsize)],
-                                              consecutive=consecutive)
+                return self.get_byte_progress(
+                    [(self.get_vod_fileindex(), self.vod_seekpos, self.vod_seekpos + self.prebuffsize)],
+                    consecutive=consecutive)
         else:
             return 0.0
 
@@ -848,7 +873,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         peers = []
         peer_infos = self.handle.get_peer_info() if self.handle and self.handle.is_valid() else []
         for peer_info in peer_infos:
-            peer_dict = {'id': peer_info.pid.to_bytes().encode('hex'),
+            peer_dict = {'id': hexlify(peer_info.pid.to_bytes()),
                          'extended_version': peer_info.client,
                          'ip': peer_info.ip[0],
                          'port': peer_info.ip[1],
@@ -870,9 +895,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                          'have': peer_info.pieces, 'speed': peer_info.remote_dl_rate,
                          'connection_type': peer_info.connection_type,
                          'seed': bool(peer_info.flags & peer_info.seed),
-                         'upload_only': bool(peer_info.flags & peer_info.upload_only),
-                         'rstate': peer_info.read_state,
-                         'wstate': peer_info.write_state}
+                         'upload_only': bool(peer_info.flags & peer_info.upload_only)}
             peers.append(peer_dict)
         return peers
 
@@ -896,7 +919,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         for announce_entry in self.handle.trackers():
             if announce_entry['url'] not in self.tracker_status:
                 try:
-                    url = unicode(announce_entry['url'])
+                    url = six.text_type(announce_entry['url'])
                     self.tracker_status[url] = [0, 'Not contacted yet']
                 except UnicodeDecodeError:
                     pass
@@ -914,7 +937,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
         result = self.tracker_status.copy()
         result['[DHT]'] = [dht_peers, 'Working' if ltsession.is_dht_running() and public else 'Disabled']
-        result['[PeX]'] = [pex_peers, 'Working' if not self.get_anon_mode() else 'Disabled']
+        result['[PeX]'] = [pex_peers, 'Working']
         return result
 
     def set_state_callback(self, usercallback):
@@ -997,15 +1020,13 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
         with self.dllock:
             if self.handle is None:
-                def schedule_create_engine(_):
-                    self.cew_scheduled = True
-                    create_engine_wrapper_deferred = self.network_create_engine_wrapper(
-                        self.pstate_for_restart, share_mode=self.get_share_mode())
-                    create_engine_wrapper_deferred.addCallback(self.session.lm.on_download_handle_created)
-
-                can_create_engine_deferred = self.can_create_engine_wrapper()
-                can_create_engine_deferred.addCallback(schedule_create_engine)
+                self.cew_scheduled = True
+                create_engine_wrapper_deferred = self.network_create_engine_wrapper(self.pstate_for_restart,
+                                                                                    share_mode=self.get_share_mode(),
+                                                                                    upload_mode=self.get_upload_mode())
+                create_engine_wrapper_deferred.addCallback(self.session.lm.on_download_handle_created)
             else:
+                self.handle.set_upload_mode(self.get_upload_mode())
                 self.handle.resume()
                 self.set_vod_mode(self.get_mode() == DLMODE_VOD)
 
@@ -1041,7 +1062,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
             filename = os.path.join(self.session.get_downloads_pstate_dir(), basename)
             if not os.path.isfile(filename):
                 resume_data = self.pstate_for_restart.get('state', 'engineresumedata') \
-                              if self.pstate_for_restart else None
+                    if self.pstate_for_restart else None
 
                 # 2. If there is no saved data for this infohash, checkpoint it without data so we do not
                 #    lose it when we crash or restart before the download becomes known.
@@ -1050,7 +1071,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     'file-version': 1,
                     'info-hash': self.tdef.get_infohash()
                 }
-                alert = type('anonymous_alert', (object, ), dict(resume_data=resume_data))
+                alert = type('anonymous_alert', (object,), dict(resume_data=resume_data))
                 self.on_save_resume_data_alert(alert)
             return succeed(None)
 
@@ -1067,7 +1088,8 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         pstate.set('state', 'version', PERSISTENTSTATE_CURRENTVERSION)
         if isinstance(self.tdef, TorrentDefNoMetainfo):
             pstate.set('state', 'metainfo', {
-                       'infohash': self.tdef.get_infohash(), 'name': self.tdef.get_name_as_unicode(), 'url': self.tdef.get_url()})
+                'infohash': self.tdef.get_infohash(), 'name': self.tdef.get_name_as_unicode(),
+                'url': self.tdef.get_url()})
         else:
             pstate.set('state', 'metainfo', self.tdef.get_metainfo())
 
@@ -1125,3 +1147,14 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
     def set_share_mode(self, share_mode):
         self.get_handle().addCallback(lambda handle: handle.set_share_mode(share_mode))
+
+    @checkHandleAndSynchronize()
+    def get_upload_mode(self):
+        return self.handle.status().upload_mode
+
+    def set_upload_mode(self, upload_mode):
+        self.get_handle().addCallback(lambda handle: handle.set_upload_mode(upload_mode))
+
+    def force_dht_announce(self):
+        if self.handle:
+            self.handle.force_dht_announce()
