@@ -3,36 +3,39 @@ A Session is a running instance of the Tribler Core and the Core's central class
 
 Author(s): Arno Bakker
 """
+from __future__ import absolute_import
+
 import errno
 import logging
 import os
 import sys
-import time
-from binascii import hexlify
+from threading import RLock
+
 from twisted.internet import threads
-from twisted.internet.defer import inlineCallbacks, fail
+from twisted.internet.defer import fail, inlineCallbacks, succeed
 from twisted.python.failure import Failure
 from twisted.python.log import addObserver
 from twisted.python.threadable import isInIOThread
 
 import Tribler.Core.permid as permid_module
-from Tribler.Core import NoDispersyRLock
 from Tribler.Core.APIImplementation.LaunchManyCore import TriblerLaunchMany
-from Tribler.Core.CacheDB.Notifier import Notifier
-from Tribler.Core.CacheDB.sqlitecachedb import SQLiteCacheDB, DB_FILE_RELATIVE_PATH, DB_DIR_NAME
 from Tribler.Core.Config.tribler_config import TriblerConfig
 from Tribler.Core.Modules.restapi.rest_manager import RESTManager
+from Tribler.Core.Notifier import Notifier
 from Tribler.Core.Upgrade.upgrade import TriblerUpgrader
 from Tribler.Core.Utilities import torrent_utils
 from Tribler.Core.Utilities.crypto_patcher import patch_crypto_be_discovery
-from Tribler.Core.exceptions import NotYetImplementedException, OperationNotEnabledByConfigurationException, \
-    DuplicateTorrentFileError
-from Tribler.Core.simpledefs import (NTFY_CHANNELCAST, NTFY_DELETE, NTFY_INSERT, NTFY_MYPREFERENCES, NTFY_PEERS,
-                                     NTFY_TORRENTS, NTFY_UPDATE, NTFY_VOTECAST, STATEDIR_DLPSTATE_DIR,
-                                     STATEDIR_WALLET_DIR, STATE_OPEN_DB, STATE_START_API, STATE_UPGRADING_READABLE,
-                                     STATE_LOAD_CHECKPOINTS, STATE_READABLE_STARTED)
+from Tribler.Core.exceptions import NotYetImplementedException, OperationNotEnabledByConfigurationException
+from Tribler.Core.simpledefs import (
+    NTFY_DELETE, NTFY_INSERT, NTFY_STARTED, NTFY_TRIBLER, NTFY_UPDATE, STATEDIR_CHANNELS_DIR, STATEDIR_DB_DIR,
+    STATEDIR_DLPSTATE_DIR, STATEDIR_WALLET_DIR, STATE_LOAD_CHECKPOINTS, STATE_READABLE_STARTED, STATE_SHUTDOWN,
+    STATE_START_API, STATE_UPGRADING_READABLE)
 from Tribler.Core.statistics import TriblerStatistics
-from Tribler.pyipv8.ipv8.util import blocking_call_on_reactor_thread
+
+try:
+    long  # pylint: disable=long-builtin
+except NameError:
+    long = int  # pylint: disable=redefined-builtin
 
 if sys.platform == 'win32':
     SOCKET_BLOCK_ERRORCODE = 10035  # WSAEWOULDBLOCK
@@ -65,15 +68,14 @@ class Session(object):
 
         self._logger = logging.getLogger(self.__class__.__name__)
 
-        self.session_lock = NoDispersyRLock()
+        self.session_lock = RLock()
 
         self.config = config or TriblerConfig()
+        self._logger.info("Session is using state directory: %s", self.config.get_state_dir())
+
         self.get_ports_in_config()
         self.create_state_directory_structure()
 
-        if not self.config.get_megacache_enabled():
-            self.config.set_torrent_checking_enabled(False)
-
         self.selected_ports = self.config.selected_ports
 
         self.init_keypair()
@@ -81,15 +83,15 @@ class Session(object):
         self.lm = TriblerLaunchMany()
         self.notifier = Notifier()
 
-        self.sqlite_db = None
         self.upgrader_enabled = True
-        self.dispersy_member = None
+        self.upgrader = None
         self.readable_status = ''  # Human-readable string to indicate the status during startup/shutdown of Tribler
 
         self.autoload_discovery = autoload_discovery
 
     def create_state_directory_structure(self):
         """Create directory structure of the state directory."""
+
         def create_dir(path):
             if not os.path.isdir(path):
                 os.makedirs(path)
@@ -98,17 +100,14 @@ class Session(object):
             create_dir(os.path.join(self.config.get_state_dir(), path))
 
         create_dir(self.config.get_state_dir())
-        create_dir(self.config.get_torrent_store_dir())
-        create_dir(self.config.get_metadata_store_dir())
-        create_in_state_dir(DB_DIR_NAME)
+        create_in_state_dir(STATEDIR_DB_DIR)
         create_in_state_dir(STATEDIR_DLPSTATE_DIR)
         create_in_state_dir(STATEDIR_WALLET_DIR)
+        create_in_state_dir(STATEDIR_CHANNELS_DIR)
 
     def get_ports_in_config(self):
         """Claim all required random ports."""
         self.config.get_libtorrent_port()
-        self.config.get_dispersy_port()
-        self.config.get_mainline_dht_port()
         self.config.get_video_server_port()
 
         self.config.get_anon_listen_port()
@@ -118,22 +117,6 @@ class Session(object):
         """
         Set parameters that depend on state_dir.
         """
-        permid_module.init()
-        # Set params that depend on state_dir
-        #
-        # 1. keypair
-        #
-        pair_filename = self.config.get_permid_keypair_filename()
-        if os.path.exists(pair_filename):
-            self.keypair = permid_module.read_keypair(pair_filename)
-        else:
-            self.keypair = permid_module.generate_keypair()
-
-            # Save keypair
-            public_key_filename = os.path.join(self.config.get_state_dir(), 'ecpub.pem')
-            permid_module.save_keypair(self.keypair, pair_filename)
-            permid_module.save_pub_key(self.keypair, public_key_filename)
-
         trustchain_pairfilename = self.config.get_trustchain_keypair_filename()
         if os.path.exists(trustchain_pairfilename):
             self.trustchain_keypair = permid_module.read_keypair_trustchain(trustchain_pairfilename)
@@ -188,6 +171,10 @@ class Session(object):
                 self._logger.error("Could not send data: socket is busy.")
                 return
 
+            if 'socket.error' in text and '[Errno 11001]' in text:
+                self._logger.error("Unable to perform DNS lookup.")
+                return
+
             if 'socket.error' in text and '[Errno 10053]' in text:
                 self._logger.error("An established connection was aborted by the software in your host machine.")
                 return
@@ -204,6 +191,10 @@ class Session(object):
                 self._logger.error("Internal Twisted response error, consider updating your Twisted version.")
                 return
 
+            if 'twisted.internet.error.AlreadyCalled' in text:
+                self._logger.error("Tried to cancel an already called event\n%s", text)
+                return
+
             # We already have a check for invalid infohash when adding a torrent, but if somehow we get this
             # error then we simply log and ignore it.
             if 'exceptions.RuntimeError: invalid info-hash' in text:
@@ -240,7 +231,7 @@ class Session(object):
 
         Locking is done by LaunchManyCore.
 
-        :param torrent_definition: a finalized TorrentDef
+        :param torrent_definition: a TorrentDef
         :param download_startup_config: a DownloadStartupConfig or None, in which case
         a new DownloadStartupConfig() is created with its default settings
         and the result becomes the runtime config of this Download
@@ -322,8 +313,6 @@ class Session(object):
             if download.get_def().get_infohash() == infohash:
                 return self.remove_download(download, remove_content, remove_state)
 
-        self.lm.remove_id(infohash)
-
     def set_download_states_callback(self, user_callback, interval=1.0):
         """
         See Download.set_state_callback. Calls user_callback with a list of
@@ -341,18 +330,6 @@ class Session(object):
         """
         self.lm.set_download_states_callback(user_callback, interval)
 
-    #
-    # Config parameters that only exist at runtime
-    #
-    def get_permid(self):
-        """
-        Returns the PermID of the Session, as determined by the
-        TriblerConfig.set_permid() parameter. A PermID is a public key.
-
-        :return: the PermID encoded in a string in DER format
-        """
-        return str(self.keypair.pub().get_der())
-
     #
     # Notification of events in the Session
     #
@@ -387,57 +364,14 @@ class Session(object):
         """
         self.notifier.remove_observer(function)
 
-    def open_dbhandler(self, subject):
-        """
-        Opens a connection to the specified database. Only the thread calling this method may
-        use this connection. The connection must be closed with close_dbhandler() when this
-        thread exits. This function is called by any thread.
-
-        ;param subject: the database to open. Must be one of the subjects specified here.
-        :return: a reference to a DBHandler class for the specified subject or
-        None when the Session was not started with megacache enabled.
-        """
-        if not self.config.get_megacache_enabled():
-            raise OperationNotEnabledByConfigurationException()
-
-        if subject == NTFY_PEERS:
-            return self.lm.peer_db
-        elif subject == NTFY_TORRENTS:
-            return self.lm.torrent_db
-        elif subject == NTFY_MYPREFERENCES:
-            return self.lm.mypref_db
-        elif subject == NTFY_VOTECAST:
-            return self.lm.votecast_db
-        elif subject == NTFY_CHANNELCAST:
-            return self.lm.channelcast_db
-        else:
-            raise ValueError(u"Cannot open DB subject: %s" % subject)
-
-    @staticmethod
-    def close_dbhandler(database_handler):
-        """Closes the given database connection."""
-        database_handler.close()
-
     def get_tribler_statistics(self):
         """Return a dictionary with general Tribler statistics."""
         return TriblerStatistics(self).get_tribler_statistics()
 
-    def get_dispersy_statistics(self):
-        """Return a dictionary with general Dispersy statistics."""
-        return TriblerStatistics(self).get_dispersy_statistics()
-
-    def get_dispersy_community_statistics(self):
-        """Return a dictionary with Dispersy communities statistics."""
-        return TriblerStatistics(self).get_dispersy_community_statistics()
-
     def get_ipv8_statistics(self):
         """Return a dictionary with IPv8 statistics."""
         return TriblerStatistics(self).get_ipv8_statistics()
 
-    def get_ipv8_overlay_statistics(self):
-        """Return a dictionary with IPv8 overlay statistics."""
-        return TriblerStatistics(self).get_ipv8_overlays_statistics()
-
     #
     # Persistence and shutdown
     #
@@ -457,19 +391,6 @@ class Session(object):
         """
         self.lm.checkpoint_downloads()
 
-    @blocking_call_on_reactor_thread
-    def start_database(self):
-        """
-        Start the SQLite database.
-        """
-        db_path = os.path.join(self.config.get_state_dir(), DB_FILE_RELATIVE_PATH)
-
-        self.sqlite_db = SQLiteCacheDB(db_path)
-        self.readable_status = STATE_OPEN_DB
-        self.sqlite_db.initialize()
-        self.sqlite_db.initial_begin()
-
-    @blocking_call_on_reactor_thread
     def start(self):
         """
         Start a Tribler session by initializing the LaunchManyCore class, opening the database and running the upgrader.
@@ -481,14 +402,26 @@ class Session(object):
             self.readable_status = STATE_START_API
             self.lm.api_manager.start()
 
-        self.start_database()
-
         if self.upgrader_enabled:
-            upgrader = TriblerUpgrader(self, self.sqlite_db)
+            self.upgrader = TriblerUpgrader(self)
             self.readable_status = STATE_UPGRADING_READABLE
-            upgrader.run()
+            upgrader_deferred = self.upgrader.run()
+        else:
+            upgrader_deferred = succeed(None)
 
-        startup_deferred = self.lm.register(self, self.session_lock)
+        def after_upgrade(_):
+            self.upgrader = None
+
+        def log_upgrader_error(failure):
+            self._logger.error("Error in Upgrader callback chain: %s", failure)
+
+        def on_tribler_started(_):
+            self.notifier.notify(NTFY_TRIBLER, NTFY_STARTED, None)
+
+        startup_deferred = upgrader_deferred. \
+            addCallbacks(lambda _: self.lm.register(self, self.session_lock), log_upgrader_error). \
+            addCallbacks(after_upgrade, log_upgrader_error). \
+            addCallback(on_tribler_started)
 
         def load_checkpoint(_):
             if self.config.get_libtorrent_enabled():
@@ -496,9 +429,14 @@ class Session(object):
                 self.load_checkpoint()
             self.readable_status = STATE_READABLE_STARTED
 
-        return startup_deferred.addCallback(load_checkpoint)
+        def start_gigachannel_manager(_):
+            # GigaChannel Manager should be started *after* resuming the downloads,
+            # because it depends on the states of torrent downloads
+            # TODO: move GigaChannel torrents into a separate session
+            if self.lm.gigachannel_manager:
+                self.lm.gigachannel_manager.start()
+        return startup_deferred.addCallback(load_checkpoint).addCallback(start_gigachannel_manager)
 
-    @blocking_call_on_reactor_thread
     def shutdown(self):
         """
         Checkpoints the session and closes it, stopping the download engine.
@@ -513,14 +451,31 @@ class Session(object):
             Continues the shutdown procedure that is dependant on the early shutdown.
             :param _: ignored parameter of the Deferred
             """
+            self.notify_shutdown_state("Saving configuration...")
             self.config.write()
+
+            self.notify_shutdown_state("Checkpointing Downloads...")
             yield self.checkpoint_downloads()
+
+            self.notify_shutdown_state("Shutting down Downloads...")
             self.lm.shutdown_downloads()
+
+            self.notify_shutdown_state("Shutting down Network...")
             self.lm.network_shutdown()
 
-            if self.sqlite_db:
-                self.sqlite_db.close()
-            self.sqlite_db = None
+            if self.lm.mds:
+                self.notify_shutdown_state("Shutting down Metadata Store...")
+                self.lm.mds.shutdown()
+
+            # We close the API manager as late as possible during shutdown.
+            if self.lm.api_manager is not None:
+                self.notify_shutdown_state("Shutting down API Manager...")
+                yield self.lm.api_manager.stop()
+            self.lm.api_manager = None
+
+        # Indicates we are shutting down core. With this environment variable set
+        # to 'TRUE', RESTManager will no longer accepts any new requests.
+        os.environ['TRIBLER_SHUTTING_DOWN'] = "TRUE"
 
         return self.lm.early_shutdown().addCallback(on_early_shutdown_complete)
 
@@ -541,69 +496,6 @@ class Session(object):
         """
         return os.path.join(self.config.get_state_dir(), STATEDIR_DLPSTATE_DIR)
 
-    def download_torrentfile(self, infohash=None, user_callback=None, priority=0):
-        """
-        Try to download the torrent file without a known source. A possible source could be the DHT.
-        If the torrent is received successfully, the user_callback method is called with the infohash as first
-        and the contents of the torrent file (bencoded dict) as second parameter. If the torrent could not
-        be obtained, the callback is not called. The torrent will have been added to the TorrentDBHandler (if enabled)
-        at the time of the call.
-
-        :param infohash: the infohash of the torrent
-        :param user_callback: a function adhering to the above spec
-        :param priority: the priority of this download
-        """
-        if not self.lm.rtorrent_handler:
-            raise OperationNotEnabledByConfigurationException()
-
-        self.lm.rtorrent_handler.download_torrent(None, infohash, user_callback=user_callback, priority=priority)
-
-    def download_torrentfile_from_peer(self, candidate, infohash=None, user_callback=None, priority=0):
-        """
-        Ask the designated peer to send us the torrent file for the torrent
-        identified by the passed infohash. If the torrent is successfully
-        received, the user_callback method is called with the infohash as first
-        and the contents of the torrent file (bencoded dict) as second parameter.
-        If the torrent could not be obtained, the callback is not called.
-        The torrent will have been added to the TorrentDBHandler (if enabled)
-        at the time of the call.
-
-        :param candidate: the designated peer
-        :param infohash: the infohash of the torrent
-        :param user_callback: a function adhering to the above spec
-        :param priority: priority of this request
-        """
-        if not self.lm.rtorrent_handler:
-            raise OperationNotEnabledByConfigurationException()
-
-        self.lm.rtorrent_handler.download_torrent(candidate, infohash, user_callback=user_callback, priority=priority)
-
-    def download_torrentmessage_from_peer(self, candidate, infohash, user_callback, priority=0):
-        """
-        Ask the designated peer to send us the torrent message for the torrent
-        identified by the passed infohash. If the torrent message is successfully
-        received, the user_callback method is called with the infohash as first
-        and the contents of the torrent file (bencoded dict) as second parameter.
-        If the torrent could not be obtained, the callback is not called.
-        The torrent will have been added to the TorrentDBHandler (if enabled)
-        at the time of the call.
-
-        :param candidate: the designated peer
-        :param infohash: the infohash of the torrent
-        :param user_callback: a function adhering to the above spec
-        :param priority: priority of this request
-        """
-        if not self.lm.rtorrent_handler:
-            raise OperationNotEnabledByConfigurationException()
-
-        self.lm.rtorrent_handler.download_torrentmessage(candidate, infohash, user_callback, priority)
-
-    def get_dispersy_instance(self):
-        if not self.config.get_dispersy_enabled():
-            raise OperationNotEnabledByConfigurationException()
-
-        return self.lm.dispersy
-
     def get_ipv8_instance(self):
         if not self.config.get_ipv8_enabled():
             raise OperationNotEnabledByConfigurationException()
@@ -632,71 +524,6 @@ class Session(object):
         """
         return self.lm.update_trackers(infohash, trackers)
 
-    def has_collected_torrent(self, infohash):
-        """
-        Checks if the given torrent infohash exists in the torrent_store database.
-
-        :param infohash: The given infohash binary
-        :return: True or False indicating if we have the torrent
-        """
-        if not self.config.get_torrent_store_enabled():
-            raise OperationNotEnabledByConfigurationException("torrent_store is not enabled")
-        return hexlify(infohash) in self.lm.torrent_store
-
-    def get_collected_torrent(self, infohash):
-        """
-        Gets the given torrent from the torrent_store database.
-
-        :param infohash: the given infohash binary
-        :return: the torrent data if exists, None otherwise
-        """
-        if not self.config.get_torrent_store_enabled():
-            raise OperationNotEnabledByConfigurationException("torrent_store is not enabled")
-        return self.lm.torrent_store.get(hexlify(infohash))
-
-    def save_collected_torrent(self, infohash, data):
-        """
-        Saves the given torrent into the torrent_store database.
-
-        :param infohash: the given infohash binary
-        :param data: the torrent file data
-        """
-        if not self.config.get_torrent_store_enabled():
-            raise OperationNotEnabledByConfigurationException("torrent_store is not enabled")
-        self.lm.torrent_store.put(hexlify(infohash), data)
-
-    def delete_collected_torrent(self, infohash):
-        """
-        Deletes the given torrent from the torrent_store database.
-
-        :param infohash: the given infohash binary
-        """
-        if not self.config.get_torrent_store_enabled():
-            raise OperationNotEnabledByConfigurationException("torrent_store is not enabled")
-
-        del self.lm.torrent_store[hexlify(infohash)]
-
-    def search_remote_torrents(self, keywords):
-        """
-        Searches for remote torrents through SearchCommunity with the given keywords.
-
-        :param keywords: the given keywords
-        :return: the number of requests made
-        """
-        if not self.config.get_torrent_search_enabled():
-            raise OperationNotEnabledByConfigurationException("torrent_search is not enabled")
-        return self.lm.search_manager.search_for_torrents(keywords)
-
-    def search_remote_channels(self, keywords):
-        """
-        Searches for remote channels through AllChannelCommunity with the given keywords.
-
-        :param keywords: the given keywords
-        """
-        if not self.config.get_channel_search_enabled():
-            raise OperationNotEnabledByConfigurationException("channel_search is not enabled")
-        self.lm.search_manager.search_for_channels(keywords)
-
     @staticmethod
     def create_torrent_file(file_path_list, params=None):
         """
@@ -717,45 +544,10 @@ class Session(object):
         :param description: description of the Channel
         :param mode: mode of the Channel ('open', 'semi-open', or 'closed')
         :return: a channel ID
-        :raises a DuplicateChannelNameError if name already exists
+        :raises a DuplicateChannelIdError if name already exists
         """
         return self.lm.channel_manager.create_channel(name, description, mode)
 
-    def add_torrent_def_to_channel(self, channel_id, torrent_def, extra_info={}, forward=True):
-        """
-        Adds a TorrentDef to a Channel.
-
-        :param channel_id: id of the Channel to add the Torrent to
-        :param torrent_def: definition of the Torrent to add
-        :param extra_info: description of the Torrent to add
-        :param forward: when True the messages are forwarded (as defined by their message
-         destination policy) to other nodes in the community. This parameter should (almost always)
-         be True, its inclusion is mostly to allow certain debugging scenarios
-        """
-        # Make sure that this new torrent_def is also in collected torrents
-        self.lm.rtorrent_handler.save_torrent(torrent_def)
-
-        channelcast_db = self.open_dbhandler(NTFY_CHANNELCAST)
-        if channelcast_db.hasTorrent(channel_id, torrent_def.infohash):
-            raise DuplicateTorrentFileError("This torrent file already exists in your channel.")
-
-        dispersy_cid = str(channelcast_db.getDispersyCIDFromChannelId(channel_id))
-        community = self.get_dispersy_instance().get_community(dispersy_cid)
-
-        community._disp_create_torrent(
-            torrent_def.infohash,
-            long(time.time()),
-            torrent_def.get_name_as_unicode(),
-            tuple(torrent_def.get_files_with_length()),
-            torrent_def.get_trackers_as_single_tuple(),
-            forward=forward)
-
-        if 'description' in extra_info:
-            desc = extra_info['description'].strip()
-            if desc != '':
-                data = channelcast_db.getTorrentFromChannelId(channel_id, torrent_def.infohash, ['ChannelTorrents.id'])
-                community.modifyTorrent(data, {'description': desc}, forward=forward)
-
     def check_torrent_health(self, infohash, timeout=20, scrape_now=False):
         """
         Checks the given torrent's health on its trackers.
@@ -765,16 +557,9 @@ class Session(object):
         :param scrape_now: flag to scrape immediately
         """
         if self.lm.torrent_checker:
-            return self.lm.torrent_checker.add_gui_request(infohash, timeout=timeout, scrape_now=scrape_now)
+            return self.lm.torrent_checker.check_torrent_health(infohash, timeout=timeout, scrape_now=scrape_now)
         return fail(Failure(RuntimeError("Torrent checker not available")))
 
-    def get_thumbnail_data(self, thumb_hash):
-        """
-        Gets the thumbnail data.
-
-        :param thumb_hash: the thumbnail SHA1 hash
-        :return: the thumbnail data
-        """
-        if not self.lm.metadata_store:
-            raise OperationNotEnabledByConfigurationException("libtorrent is not enabled")
-        return self.lm.rtorrent_handler.get_metadata(thumb_hash)
+    def notify_shutdown_state(self, state):
+        self._logger.info("Tribler shutdown state notification:%s", state)
+        self.notifier.notify(NTFY_TRIBLER, STATE_SHUTDOWN, None, state)
