@@ -2,12 +2,15 @@ from __future__ import absolute_import
 
 import binascii
 import os
+import shutil
 
 import libtorrent as lt
+from libtorrent import bdecode, bencode
 
+from six import text_type
 from six.moves import xrange
 
-from twisted.internet.defer import Deferred, succeed
+from twisted.internet.defer import Deferred, inlineCallbacks, succeed
 
 from Tribler.Core.DownloadConfig import DownloadStartupConfig
 from Tribler.Core.Libtorrent.LibtorrentDownloadImpl import LibtorrentDownloadImpl
@@ -16,7 +19,7 @@ from Tribler.Core.Utilities.configparser import CallbackConfigParser
 from Tribler.Core.Utilities.torrent_utils import get_info_from_handle
 from Tribler.Core.simpledefs import DLMODE_VOD, DLSTATUS_DOWNLOADING
 from Tribler.Test.Core.base_test import MockObject, TriblerCoreTest
-from Tribler.Test.common import TESTS_DATA_DIR
+from Tribler.Test.common import TESTS_DATA_DIR, TORRENT_UBUNTU_FILE
 from Tribler.Test.test_as_server import TestAsServer
 from Tribler.Test.tools import trial_timeout
 
@@ -29,11 +32,7 @@ class TestLibtorrentDownloadImpl(TestAsServer):
     def setUpPreSession(self):
         super(TestLibtorrentDownloadImpl, self).setUpPreSession()
         self.config.set_torrent_checking_enabled(False)
-        self.config.set_megacache_enabled(True)
-        self.config.set_dispersy_enabled(False)
         self.config.set_tunnel_community_enabled(False)
-        self.config.set_mainline_dht_enabled(False)
-        self.config.set_torrent_collecting_enabled(False)
         self.config.set_libtorrent_enabled(True)
         self.config.set_video_server_enabled(False)
 
@@ -45,8 +44,6 @@ class TestLibtorrentDownloadImpl(TestAsServer):
         sourcefn = os.path.join(TESTS_DATA_DIR, 'video.avi')
         tdef.add_content(sourcefn)
         tdef.set_tracker("http://localhost/announce")
-        tdef.finalize()
-
         torrentfn = os.path.join(self.session.config.get_state_dir(), "gen.torrent")
         tdef.save(torrentfn)
 
@@ -138,17 +135,16 @@ class TestLibtorrentDownloadImpl(TestAsServer):
         return test_deferred
 
     @trial_timeout(20)
+    @inlineCallbacks
     def test_multifile_torrent(self):
+        # Achtung! This test is completely and utterly broken, as is the whole libtorrent wrapper!
+        # Don't try to understand it, it is a legacy thing!
+
         tdef = TorrentDef()
 
-        dn = os.path.join(TESTS_DATA_DIR, "contentdir")
-        tdef.add_content(dn, "dirintorrent")
-
-        fn = os.path.join(TESTS_DATA_DIR, "video.avi")
-        tdef.add_content(fn, os.path.join("dirintorrent", "video.avi"))
-
+        tdef.add_content(os.path.join(TESTS_DATA_DIR, "video.avi"))
         tdef.set_tracker("http://tribler.org/announce")
-        tdef.finalize()
+        tdef.save()
 
         impl = LibtorrentDownloadImpl(self.session, tdef)
         # Override the add_torrent because it will be called
@@ -167,13 +163,16 @@ class TestLibtorrentDownloadImpl(TestAsServer):
         fake_status.share_mode = False
         # Create a dummy download config
         impl.dlconfig = DownloadStartupConfig().dlconfig.copy()
-        # Create a dummy pstate
+
         pstate = CallbackConfigParser()
         pstate.add_section("state")
-        test_dict = dict()
-        test_dict["a"] = "b"
-        pstate.set("state", "engineresumedata", test_dict)
-        return impl.network_create_engine_wrapper(pstate)
+        pstate.set("state", "engineresumedata", {"save_path": text_type(os.path.abspath(self.state_dir))})
+        yield impl.network_create_engine_wrapper(pstate)
+
+        pstate = CallbackConfigParser()
+        pstate.add_section("state")
+        pstate.set("state", "engineresumedata", {"save_path": "some_local_dir"})
+        yield impl.network_create_engine_wrapper(pstate)
 
     @trial_timeout(10)
     def test_save_resume(self):
@@ -473,14 +472,16 @@ class TestLibtorrentDownloadImplNoSession(TriblerCoreTest):
 
         self.libtorrent_download_impl.handle.trackers = lambda: []
         self.libtorrent_download_impl.handle.save_resume_data = lambda: None
-        torrent_dict = {'name': 'test', 'piece length': 42, 'pieces': '', 'files': []}
-        get_info_from_handle(self.libtorrent_download_impl.handle).metadata = lambda: lt.bencode(torrent_dict)
+        self.libtorrent_download_impl.handle.rename_file = lambda *_: None
+        with open(os.path.join(TESTS_DATA_DIR, "bak_single.torrent"), mode='rb') as torrent_file:
+            encoded_metainfo = torrent_file.read()
+        decoded_metainfo = bdecode(encoded_metainfo)
+        get_info_from_handle(self.libtorrent_download_impl.handle).metadata = lambda: bencode(decoded_metainfo['info'])
         get_info_from_handle(self.libtorrent_download_impl.handle).files = lambda: [mocked_file]
 
         self.libtorrent_download_impl.checkpoint = mocked_checkpoint
         self.libtorrent_download_impl.session = MockObject()
         self.libtorrent_download_impl.session.lm = MockObject()
-        self.libtorrent_download_impl.session.lm.rtorrent_handler = None
         self.libtorrent_download_impl.session.lm.torrent_db = None
         self.libtorrent_download_impl.handle.save_path = lambda: None
         self.libtorrent_download_impl.handle.prioritize_files = lambda _: None
@@ -496,6 +497,7 @@ class TestLibtorrentDownloadImplNoSession(TriblerCoreTest):
         """
         unwanted_dir = os.path.join(self.getStateDir(), '.unwanted')
         os.mkdir(unwanted_dir)
+        shutil.copyfile(TORRENT_UBUNTU_FILE, os.path.join(unwanted_dir, "test.txt"))
         self.libtorrent_download_impl.handle.save_path = lambda: self.getStateDir()
         self.libtorrent_download_impl.handle.file_priorities = lambda: [1]
         self.libtorrent_download_impl.orig_files = ['test']
@@ -514,7 +516,6 @@ class TestLibtorrentDownloadImplNoSession(TriblerCoreTest):
         self.libtorrent_download_impl.handle.get_torrent_info = lambda: None
         self.libtorrent_download_impl.on_metadata_received_alert(None)
 
-
     def test_metadata_received_invalid_torrent_with_value_error(self):
         """
         Testing whether the right operations happen when we receive metadata but the torrent info is invalid and throws
@@ -529,8 +530,7 @@ class TestLibtorrentDownloadImplNoSession(TriblerCoreTest):
         # The line below should trigger Value Error
         self.libtorrent_download_impl.handle.trackers = lambda: [{'url': 'no-DHT'}]
 
-        torrent_dict = {'name': 'test', 'piece length': 42, 'pieces': '', 'files': []}
-        get_info_from_handle(self.libtorrent_download_impl.handle).metadata = lambda: lt.bencode(torrent_dict)
+        get_info_from_handle(self.libtorrent_download_impl.handle).metadata = lambda: lt.bencode({})
         get_info_from_handle(self.libtorrent_download_impl.handle).files = lambda: [mocked_file]
 
         self.libtorrent_download_impl.checkpoint = mocked_checkpoint
@@ -688,7 +688,7 @@ class TestLibtorrentDownloadImplNoSession(TriblerCoreTest):
         self.libtorrent_download_impl.on_torrent_finished_alert(None)
 
         has_priorities_task = False
-        for task_name in self.libtorrent_download_impl._pending_tasks.iterkeys():
+        for task_name in self.libtorrent_download_impl._pending_tasks:
             if 'reset_priorities' in task_name:
                 has_priorities_task = True
         self.assertTrue(has_priorities_task)
