@@ -1,22 +1,26 @@
-import binascii
+from __future__ import absolute_import
+
 import os
 import shutil
 import tempfile
+from binascii import hexlify, unhexlify
 
+import libtorrent
 from libtorrent import bencode
 
+from twisted.internet import reactor
+from twisted.internet.defer import Deferred, inlineCallbacks
 from twisted.internet.task import deferLater
 
-from Tribler.Test.tools import trial_timeout
-from twisted.internet.defer import inlineCallbacks, Deferred
-from twisted.internet import reactor
-
-from Tribler.Core.CacheDB.Notifier import Notifier
 from Tribler.Core.Libtorrent.LibtorrentDownloadImpl import LibtorrentDownloadImpl
 from Tribler.Core.Libtorrent.LibtorrentMgr import LibtorrentMgr
+from Tribler.Core.Notifier import Notifier
+from Tribler.Core.TorrentDef import TorrentDef
 from Tribler.Core.exceptions import TorrentFileException
 from Tribler.Test.Core.base_test import MockObject
+from Tribler.Test.common import TESTS_DATA_DIR
 from Tribler.Test.test_as_server import AbstractServer
+from Tribler.Test.tools import trial_timeout
 
 
 class TestLibtorrentMgr(AbstractServer):
@@ -83,25 +87,11 @@ class TestLibtorrentMgr(AbstractServer):
         ltsession = self.ltmgr.get_session(0)
         self.assertTrue(ltsession)
 
-    def test_get_metainfo_not_ready(self):
-        """
-        Testing the metainfo fetching method when the DHT is not ready
-        """
-        self.ltmgr.initialize()
-        self.assertFalse(self.ltmgr.get_metainfo("a" * 20, None))
-
     @trial_timeout(20)
-    def test_get_metainfo(self):
+    def test_get_metainfo_valid_metadata(self):
         """
-        Testing the metainfo fetching method
+        Testing the get_metainfo method when the handle has valid metadata immediately
         """
-        test_deferred = Deferred()
-
-        def metainfo_cb(metainfo):
-            self.assertEqual(metainfo, {'info': {'pieces': ['a']}, 'leechers': 0,
-                                        'nodes': [], 'seeders': 0, 'initial peers': []})
-            test_deferred.callback(None)
-
         infohash = "a" * 20
 
         self.ltmgr.initialize()
@@ -118,121 +108,190 @@ class TestLibtorrentMgr(AbstractServer):
         self.ltmgr.ltsession_metainfo.add_torrent = lambda *_: fake_handle
         self.ltmgr.ltsession_metainfo.remove_torrent = lambda *_: None
 
-        fake_alert = type('lt.metadata_received_alert', (object,), dict(handle=fake_handle))
-        self.ltmgr.ltsession_metainfo.pop_alerts = lambda: [fake_alert]
+        def verify_metainfo(metainfo):
+            self.assertEqual(metainfo, {'info': {'pieces': ['a']}, 'leechers': 0,
+                                        'nodes': [], 'seeders': 0})
 
-        self.ltmgr.is_dht_ready = lambda: True
-        self.ltmgr.get_metainfo(infohash.decode('hex'), metainfo_cb)
+        return self.ltmgr.get_metainfo(unhexlify(infohash)).addCallback(verify_metainfo)
 
-        return test_deferred
+    @trial_timeout(20)
+    def test_get_metainfo_add_fail(self):
+        """
+        Test whether we try to add a torrent again if the atp is rejected
+        """
+        infohash = "a" * 20
+
+        self.ltmgr.initialize()
+
+        def mock_add_torrent(atp):
+            if isinstance(atp['info_hash'], libtorrent.sha1_hash):
+                raise TypeError
+            else:
+                torrent_info = MockObject()
+                torrent_info.metadata = lambda: bencode({'pieces': ['a']})
+                torrent_info.trackers = lambda: []
+
+                fake_handle = MockObject()
+                fake_handle.is_valid = lambda: True
+                fake_handle.has_metadata = lambda: True
+                fake_handle.get_peer_info = lambda: []
+                fake_handle.torrent_file = lambda: torrent_info
+                return fake_handle
+
+        self.ltmgr.ltsession_metainfo.add_torrent = mock_add_torrent
+        self.ltmgr.ltsession_metainfo.remove_torrent = lambda *_: None
+        return self.ltmgr.get_metainfo(unhexlify(infohash), timeout=0.1)
+
+    @trial_timeout(20)
+    def test_get_metainfo_duplicate_request(self):
+        """
+        Test whether the same request is returned when invoking get_metainfo twice with the same infohash
+        """
+        infohash = "a" * 20
+
+        self.ltmgr.initialize()
+
+        torrent_info = MockObject()
+        torrent_info.metadata = lambda: bencode({'pieces': ['a']})
+        torrent_info.trackers = lambda: []
+
+        fake_handle = MockObject()
+        fake_handle.is_valid = lambda: True
+        fake_handle.has_metadata = lambda: False
+        fake_handle.get_peer_info = lambda: []
+        fake_handle.torrent_file = lambda: torrent_info
+        self.ltmgr.ltsession_metainfo.add_torrent = lambda *_: fake_handle
+        self.ltmgr.ltsession_metainfo.remove_torrent = lambda *_: None
+
+        self.ltmgr.get_metainfo(infohash)
+        self.ltmgr.get_metainfo(infohash)
+
+        self.assertEqual(len(self.ltmgr.metainfo_requests[infohash][1]), 2)
 
     @trial_timeout(20)
     def test_get_metainfo_cache(self):
         """
-        Testing metainfo caching
+        Testing whether cached metainfo is returned, if available
         """
-        test_deferred = Deferred()
+        self.ltmgr.initialize()
+        self.ltmgr.metainfo_cache["a" * 20] = {'meta_info': 'test'}
 
-        def metainfo_cb(metainfo):
+        def verify_cached_metainfo(metainfo):
             self.assertEqual(metainfo, "test")
-            test_deferred.callback(None)
 
-        self.ltmgr.initialize()
-        self.ltmgr.is_dht_ready = lambda: True
-        self.ltmgr.metainfo_cache[("a" * 20).encode('hex')] = {'meta_info': 'test'}
-        self.ltmgr.get_metainfo("a" * 20, metainfo_cb)
-
-        return test_deferred
-
-    @trial_timeout(20)
-    def test_got_metainfo(self):
-        """
-        Testing whether the callback is correctly invoked when we received metainfo
-        """
-        test_deferred = Deferred()
-        self.ltmgr.initialize()
-
-        def metainfo_cb(metainfo):
-            self.assertDictEqual(metainfo, {'info': {'pieces': ['a']}, 'leechers': 0,
-                                            'nodes': [], 'seeders': 0, 'initial peers': []})
-            test_deferred.callback(None)
-
-        fake_handle = MockObject()
-        torrent_info = MockObject()
-        torrent_info.metadata = lambda: bencode({'pieces': ['a']})
-        torrent_info.trackers = lambda: []
-        fake_handle.get_peer_info = lambda: []
-        fake_handle.torrent_file = lambda: torrent_info
-
-        self.ltmgr.ltsession_metainfo.remove_torrent = lambda *_: None
-
-        self.ltmgr.metainfo_requests['a' * 20] = {
-            'handle': fake_handle,
-            'timeout_callbacks': [],
-            'callbacks': [metainfo_cb],
-            'notify': False
-        }
-        self.ltmgr.got_metainfo("a" * 20)
-
-        return test_deferred
-
-    @trial_timeout(20)
-    def test_got_metainfo_timeout(self):
-        """
-        Testing whether the callback is correctly invoked when we received metainfo after timeout
-        """
-        test_deferred = Deferred()
-
-        def metainfo_timeout_cb(metainfo):
-            self.assertEqual(metainfo, 'a' * 20)
-            test_deferred.callback(None)
-
-        fake_handle = MockObject()
-
-        self.ltmgr.initialize()
-        self.ltmgr.metainfo_requests[('a' * 20).encode('hex')] = {'handle': fake_handle,
-                                                                  'timeout_callbacks': [metainfo_timeout_cb],
-                                                                  'callbacks': [],
-                                                                  'notify': True}
-        self.ltmgr.ltsession_metainfo.remove_torrent = lambda _dummy1, _dummy2: None
-        self.ltmgr.got_metainfo(('a' * 20).encode('hex'), timeout=True)
-
-        return test_deferred
+        return self.ltmgr.get_metainfo("a" * 20).addCallback(verify_cached_metainfo)
 
     @trial_timeout(20)
     def test_get_metainfo_with_already_added_torrent(self):
         """
         Testing metainfo fetching for a torrent which is already in session.
-        got_metainfo() should be called with timeout=False.
         """
-        magnet_link = "magnet:?xt=urn:btih:f72636475a375653083e49d501601675ce3e6619&dn=ubuntu-16.04.3-server-i386.iso"
+        sample_torrent = os.path.join(TESTS_DATA_DIR, "bak_single.torrent")
+        torrent_def = TorrentDef.load(sample_torrent)
 
-        test_deferred = Deferred()
-
-        def fake_got_metainfo(_, timeout):
-            self.assertFalse(timeout, "Timeout should not be True")
-            test_deferred.callback(None)
+        torrent_info = MockObject()
+        torrent_info.metadata = lambda: bencode(torrent_def.get_metainfo())
+        torrent_info.trackers = lambda: []
 
         mock_handle = MockObject()
-        mock_handle.info_hash = lambda: 'a' * 20
         mock_handle.is_valid = lambda: True
         mock_handle.has_metadata = lambda: True
+        mock_handle.torrent_file = lambda: torrent_info
+        mock_handle.get_peer_info = lambda: []
+
+        download_impl = MockObject()
+        download_impl.handle = mock_handle
+
+        hex_infohash = hexlify(torrent_def.infohash)
 
         mock_ltsession = MockObject()
-        mock_ltsession.add_torrent = lambda _: mock_handle
-        mock_ltsession.find_torrent = lambda _: mock_handle
-        mock_ltsession.get_torrents = lambda: []
-        mock_ltsession.start_upnp = lambda: None
-        mock_ltsession.stop_upnp = lambda: None
-        mock_ltsession.save_state = lambda: None
+        self.ltmgr.initialize()
+        self.ltmgr.torrents[hex_infohash] = (download_impl, mock_ltsession)
 
-        self.ltmgr.ltsession_metainfo = mock_ltsession
-        self.ltmgr.metadata_tmpdir = tempfile.mkdtemp(suffix=u'tribler_metainfo_tmpdir')
-        self.ltmgr.is_dht_ready = lambda: True
-        self.ltmgr.got_metainfo = fake_got_metainfo
+        def verify_metainfo(metainfo):
+            self.assertTrue(metainfo)
 
-        self.ltmgr.get_metainfo(magnet_link, lambda _: None)
-        return test_deferred
+        self.ltmgr.ltsession_metainfo.remove_torrent = lambda *_: None
+        return self.ltmgr.get_metainfo(torrent_def.infohash).addCallback(verify_metainfo)
+
+    @trial_timeout(20)
+    def test_check_metainfo(self):
+        """
+        Testing whether the check_metainfo method is doing the correct things
+        """
+        self.ltmgr.initialize()
+
+        fake_handle = MockObject()
+        fake_handle.is_valid = lambda: True
+        fake_handle.has_metadata = lambda: True
+
+        tracker1 = MockObject()
+        tracker1.url = 'http://test1.com'
+        tracker2 = MockObject()
+        tracker2.url = 'http://test2.com'
+
+        torrent_info = MockObject()
+        torrent_info.metadata = lambda: bencode({'pieces': ['a']})
+        torrent_info.trackers = lambda: [tracker1, tracker2]
+
+        seed_peer = MockObject()
+        seed_peer.ip = "127.0.0.1"
+        seed_peer.progress = 1
+        leech_peer = MockObject()
+        leech_peer.ip = "127.0.0.1"
+        leech_peer.progress = 0.5
+
+        fake_handle.get_peer_info = lambda: [seed_peer, leech_peer]
+        fake_handle.torrent_file = lambda: torrent_info
+
+        self.ltmgr.ltsession_metainfo.remove_torrent = lambda *_: None
+
+        test_deferred = Deferred()
+        self.ltmgr.metainfo_requests['a' * 20] = (fake_handle, [test_deferred])
+        self.ltmgr.check_metainfo(hexlify("a" * 20))
+
+        def verify_metainfo(metainfo):
+            expected = {
+                'info': {
+                    'pieces': ['a']
+                },
+                'announce': 'http://test1.com',
+                'announce-list': [['http://test1.com', 'http://test2.com']],
+                'leechers': 1,
+                'seeders': 1
+            }
+            self.assertDictEqual(metainfo, expected)
+
+        return test_deferred.addCallback(verify_metainfo)
+
+    @trial_timeout(20)
+    def test_check_metainfo_no_request(self):
+        """
+        Test whether None is returned as metainfo if there is no pending request
+        """
+        self.ltmgr.initialize()
+        self.assertFalse(self.ltmgr.check_metainfo(hexlify("a" * 20)))
+
+    @trial_timeout(20)
+    def test_check_metainfo_invalid_handle(self):
+        """
+        Test whether None is returned as metainfo if the handle is invalid
+        """
+        self.ltmgr.initialize()
+
+        fake_handle = MockObject()
+        fake_handle.is_valid = lambda: False
+        fake_handle.has_metadata = lambda: True
+
+        test_deferred = Deferred()
+        self.ltmgr.metainfo_requests['a' * 20] = (fake_handle, [test_deferred])
+
+        def verify_metainfo(metainfo):
+            self.assertFalse(metainfo)
+
+        self.ltmgr.check_metainfo(hexlify('a' * 20))
+
+        return test_deferred.addCallback(verify_metainfo)
 
     @trial_timeout(20)
     def test_add_torrent(self):
@@ -304,6 +363,13 @@ class TestLibtorrentMgr(AbstractServer):
             lambda handle: self.assertEqual(handle, mock_handle)
         )
 
+    def test_add_torrent_no_ti_url(self):
+        """
+        Test whether a ValueError is raised if we try to add a torrent without infohash or url
+        """
+        self.ltmgr.initialize()
+        self.assertRaises(ValueError, self.ltmgr.add_torrent, None, {})
+
     def test_remove_invalid_torrent(self):
         """
         Tests a successful removal status of torrents without a handle
@@ -336,14 +402,6 @@ class TestLibtorrentMgr(AbstractServer):
 
         self.assertNotIn('0'*20, self.ltmgr.torrents)
 
-    def test_start_download_corrupt(self):
-        """
-        Testing whether starting the download of a corrupt torrent file raises an exception
-        """
-        self.ltmgr.metadata_tmpdir = tempfile.mkdtemp(suffix=u'tribler_metainfo_tmpdir')
-        corrupt_file = os.path.join(self.LIBTORRENT_FILES_DIR, 'corrupt_torrent.torrent')
-        self.assertRaises(TorrentFileException, self.ltmgr.start_download, torrentfilename=corrupt_file)
-
     def test_start_download_duplicate(self):
         """
         Test the starting of a download when there are no new trackers
@@ -419,7 +477,7 @@ class TestLibtorrentMgr(AbstractServer):
 
         download = self.ltmgr.start_download_from_magnet("magnet:?xt=urn:btih:" + ('1'*40))
 
-        basename = binascii.hexlify(download.get_def().get_infohash()) + '.state'
+        basename = hexlify(download.get_def().get_infohash()) + '.state'
         filename = os.path.join(download.session.get_downloads_pstate_dir(), basename)
 
         self.assertTrue(os.path.isfile(filename))
