@@ -5,26 +5,25 @@ Every node has a chain and these chains intertwine by blocks shared by chains.
 """
 from __future__ import absolute_import
 
-from binascii import hexlify, unhexlify
 import logging
 import random
 import struct
+from binascii import hexlify, unhexlify
 from functools import wraps
 from threading import RLock
 
-from six.moves import xrange
 from twisted.internet import reactor
-from twisted.internet.defer import Deferred, succeed, fail
+from twisted.internet.defer import Deferred, fail, inlineCallbacks, maybeDeferred, returnValue, succeed
 from twisted.internet.task import LoopingCall
 
-from ...attestation.trustchain.settings import TrustChainSettings
-from .block import TrustChainBlock, ValidationResult, EMPTY_PK, GENESIS_SEQ, UNKNOWN_SEQ, ANY_COUNTERPARTY_PK
-from .caches import CrawlRequestCache, HalfBlockSignCache, IntroCrawlTimeout, ChainCrawlCache
+from .block import ANY_COUNTERPARTY_PK, EMPTY_PK, GENESIS_SEQ, TrustChainBlock, UNKNOWN_SEQ, ValidationResult
+from .caches import ChainCrawlCache, CrawlRequestCache, HalfBlockSignCache, IntroCrawlTimeout
 from .database import TrustChainDB
+from .payload import *
+from ...attestation.trustchain.settings import TrustChainSettings
 from ...community import Community
 from ...lazy_community import lazy_wrapper, lazy_wrapper_unsigned, lazy_wrapper_unsigned_wd
 from ...messaging.payload_headers import BinMemberAuthenticationPayload, GlobalTimeDistributionPayload
-from .payload import *
 from ...peer import Peer
 from ...requestcache import RandomNumberCache, RequestCache
 from ...util import addCallback
@@ -48,6 +47,7 @@ class TrustChainCommunity(Community):
     master_peer = Peer(unhexlify("4c69624e61434c504b3a5730f52156615ecbcedb36c442992ea8d3c26b418edd8bd00e01dce26028cd"
                                  "1ebe5f7dce59f4ed59f8fcee268fd7f1c6dc2fa2af8c22e3170e00cdecca487745"))
 
+    UNIVERSAL_BLOCK_LISTENER = b'UNIVERSAL_BLOCK_LISTENER'
     DB_CLASS = TrustChainDB
     DB_NAME = 'trustchain'
     version = b'\x02'
@@ -60,7 +60,7 @@ class TrustChainCommunity(Community):
         super(TrustChainCommunity, self).__init__(*args, **kwargs)
         self.request_cache = RequestCache()
         self.logger = logging.getLogger(self.__class__.__name__)
-        self.persistence = self.DB_CLASS(working_directory, db_name)
+        self.persistence = self.DB_CLASS(working_directory, db_name, self.my_peer.public_key.key_to_bin())
         self.relayed_broadcasts = []
         self.logger.debug("The trustchain community started with Public Key: %s",
                           hexlify(self.my_peer.public_key.key_to_bin()))
@@ -114,19 +114,21 @@ class TrustChainCommunity(Community):
 
         return self.listeners_map[block_type][0].BLOCK_CLASS
 
+    @inlineCallbacks
     def should_sign(self, block):
         """
         Return whether we should sign the block in the passed message.
         @param block: the block we want to sign or not.
         """
         if block.type not in self.listeners_map:
-            return False  # There are no listeners for this block
+            returnValue(False)  # There are no listeners for this block
 
         for listener in self.listeners_map[block.type]:
-            if listener.should_sign(block):
-                return True
+            should_sign = yield maybeDeferred(listener.should_sign, block)
+            if should_sign:
+                returnValue(True)
 
-        return False
+        returnValue(False)
 
     def send_block(self, block, address=None, ttl=1):
         """
@@ -220,11 +222,11 @@ class TrustChainCommunity(Community):
             "should be that reserved for any counterpary."
         assert transaction is None and linked is not None or transaction is not None and linked is None, \
             "Either provide a linked block or a transaction, not both %s, %s" % (peer, self.my_peer)
-        assert additional_info is None or additional_info is not None and linked is not None and \
-               transaction is None and peer == self.my_peer and public_key == linked.public_key, \
+        assert (additional_info is None or additional_info is not None and linked is not None
+                and transaction is None and peer == self.my_peer and public_key == linked.public_key), \
             "Either no additional info is provided or one provides it for a linked block"
-        assert linked is None or linked.link_public_key == self.my_peer.public_key.key_to_bin() or \
-               linked.link_public_key == ANY_COUNTERPARTY_PK, "Cannot counter sign block not addressed to self"
+        assert (linked is None or linked.link_public_key == self.my_peer.public_key.key_to_bin()
+                or linked.link_public_key == ANY_COUNTERPARTY_PK), "Cannot counter sign block not addressed to self"
         assert linked is None or linked.link_sequence_number == UNKNOWN_SEQ, \
             "Cannot counter sign block that is not a request"
         assert transaction is None or isinstance(transaction, dict), "Transaction should be a dictionary"
@@ -254,7 +256,7 @@ class TrustChainCommunity(Community):
 
         # This is a source block with no counterparty
         if not peer and public_key == ANY_COUNTERPARTY_PK:
-            if self.settings.broadcast_blocks:
+            if block.type not in self.settings.block_types_bc_disabled:
                 self.send_block(block)
             return succeed((block, None))
 
@@ -262,12 +264,12 @@ class TrustChainCommunity(Community):
         self.send_block(block, address=peer.address)
 
         # We broadcast the block in the network if we initiated a transaction
-        if self.settings.broadcast_blocks and not linked:
+        if block.type not in self.settings.block_types_bc_disabled and not linked:
             self.send_block(block)
 
         if peer == self.my_peer:
             # We created a self-signed block
-            if self.settings.broadcast_blocks:
+            if block.type not in self.settings.block_types_bc_disabled:
                 self.send_block(block)
 
             return succeed((block, None)) if public_key == ANY_COUNTERPARTY_PK else succeed((block, linked))
@@ -278,7 +280,7 @@ class TrustChainCommunity(Community):
             return sign_deferred
         else:
             # We return a deferred that fires immediately with both half blocks.
-            if self.settings.broadcast_blocks:
+            if block.type not in self.settings.block_types_bc_disabled:
                 self.send_block_pair(linked, block)
 
             return succeed((linked, block))
@@ -291,7 +293,7 @@ class TrustChainCommunity(Community):
         """
         peer = Peer(payload.public_key, source_address)
         block = self.get_block_class(payload.type).from_payload(payload, self.serializer)
-        self.process_half_block(block, peer)
+        self.process_half_block(block, peer).addErrback(lambda _: None)
 
     @synchronized
     @lazy_wrapper_unsigned(GlobalTimeDistributionPayload, HalfBlockBroadcastPayload)
@@ -349,7 +351,12 @@ class TrustChainCommunity(Community):
         """
         Notify listeners of a specific new block.
         """
-        if block.type not in self.listeners_map or self.shutting_down:
+        # Call the listeners associated to the universal block, if there are any
+        for listener in self.listeners_map.get(self.UNIVERSAL_BLOCK_LISTENER, []):
+            listener.received_block(block)
+
+        # Avoid proceeding any further if the type of the block coincides with the UNIVERSAL_BLOCK_LISTENER
+        if block.type not in self.listeners_map or self.shutting_down or block.type == self.UNIVERSAL_BLOCK_LISTENER:
             return
 
         for listener in self.listeners_map[block.type]:
@@ -363,7 +370,7 @@ class TrustChainCommunity(Community):
         validation = self.validate_persist_block(blk)
         self.logger.info("Block validation result %s, %s, (%s)", validation[0], validation[1], blk)
         if validation[0] == ValidationResult.invalid:
-            return
+            return fail(RuntimeError("Block could not be validated: %s, %s" % (validation[0], validation[1])))
 
         # Check if we are waiting for this signature response
         link_block_id_int = int(hexlify(blk.linked_block_id), 16) % 100000000
@@ -374,45 +381,50 @@ class TrustChainCommunity(Community):
             reactor.callFromThread(cache.sign_deferred.callback, (blk, self.persistence.get_linked(blk)))
 
         # Is this a request, addressed to us, and have we not signed it already?
-        if blk.link_sequence_number != UNKNOWN_SEQ or \
-                        blk.link_public_key != self.my_peer.public_key.key_to_bin() or \
-                        self.persistence.get_linked(blk) is not None:
-            return
+        if (blk.link_sequence_number != UNKNOWN_SEQ
+                or blk.link_public_key != self.my_peer.public_key.key_to_bin()
+                or self.persistence.get_linked(blk) is not None):
+            return succeed(None)
 
         self.logger.info("Received request block addressed to us (%s)", blk)
 
+        def on_should_sign_outcome(should_sign):
+            if not should_sign:
+                self.logger.info("Not signing block %s", blk)
+                return succeed(None)
+
+            # It is important that the request matches up with its previous block, gaps cannot be tolerated at
+            # this point. We already dropped invalids, so here we delay this message if the result is partial,
+            # partial_previous or no-info. We send a crawl request to the requester to (hopefully) close the gap
+            if (validation[0] == ValidationResult.partial_previous or validation[0] == ValidationResult.partial
+                    or validation[0] == ValidationResult.no_info) and self.settings.validation_range > 0:
+                self.logger.info("Request block could not be validated sufficiently, crawling requester. %s",
+                                 validation)
+                # Note that this code does not cover the scenario where we obtain this block indirectly.
+                if not self.request_cache.has(u"crawl", blk.hash_number):
+                    crawl_deferred = self.send_crawl_request(peer,
+                                                             blk.public_key,
+                                                             max(GENESIS_SEQ, (blk.sequence_number
+                                                                               - self.settings.validation_range)),
+                                                             max(GENESIS_SEQ, blk.sequence_number - 1),
+                                                             for_half_block=blk)
+                    return addCallback(crawl_deferred, lambda _: self.process_half_block(blk, peer))
+            else:
+                return self.sign_block(peer, linked=blk)
+
         # determine if we want to sign this block
-        if not self.should_sign(blk):
-            self.logger.info("Not signing block %s", blk)
-            return
+        return addCallback(self.should_sign(blk), on_should_sign_outcome)
 
-        # It is important that the request matches up with its previous block, gaps cannot be tolerated at
-        # this point. We already dropped invalids, so here we delay this message if the result is partial,
-        # partial_previous or no-info. We send a crawl request to the requester to (hopefully) close the gap
-        if (validation[0] == ValidationResult.partial_previous or validation[0] == ValidationResult.partial or \
-                        validation[0] == ValidationResult.no_info) and self.settings.validation_range > 0:
-            self.logger.info("Request block could not be validated sufficiently, crawling requester. %s",
-                             validation)
-            # Note that this code does not cover the scenario where we obtain this block indirectly.
-            if not self.request_cache.has(u"crawl", blk.hash_number):
-                crawl_deferred = self.send_crawl_request(peer,
-                                                         blk.public_key,
-                                                         max(GENESIS_SEQ, (blk.sequence_number -
-                                                                           self.settings.validation_range)),
-                                                         max(GENESIS_SEQ, blk.sequence_number - 1),
-                                                         for_half_block=blk)
-                addCallback(crawl_deferred, lambda _: self.process_half_block(blk, peer))
-        else:
-            self.sign_block(peer, linked=blk)
-
-    def crawl_chain(self, peer, latest_block_num=None):
+    def crawl_chain(self, peer, latest_block_num=0):
         """
         Crawl the whole chain of a specific peer.
         :param latest_block_num: The latest block number of the peer in question, if available.
         """
-        cache = ChainCrawlCache(self, peer, known_chain_length=latest_block_num)
+        crawl_deferred = Deferred()
+        cache = ChainCrawlCache(self, peer, crawl_deferred, known_chain_length=latest_block_num)
         self.request_cache.add(cache)
         reactor.callFromThread(self.send_next_partial_chain_crawl_request, cache)
+        return crawl_deferred
 
     def crawl_lowest_unknown(self, peer, latest_block_num=None):
         """
@@ -459,6 +471,7 @@ class TrustChainCommunity(Community):
         elif cache.current_request_attempts == 3:
             # We already tried the same request three times, bail out
             self.request_cache.pop(u"chaincrawl", cache.number)
+            cache.crawl_deferred.callback(None)
             return
 
         cache.current_request_attempts += 1
@@ -472,13 +485,30 @@ class TrustChainCommunity(Community):
         :param cache: The cache that stores progress regarding the chain crawl.
         """
         lowest_unknown = self.persistence.get_lowest_sequence_number_unknown(cache.peer.public_key.key_to_bin())
-        if cache.known_chain_length >= 0 and cache.known_chain_length == lowest_unknown - 1:
+        if cache.known_chain_length and cache.known_chain_length == lowest_unknown - 1:
+            # At this point, we have all the blocks we need
             self.request_cache.pop(u"chaincrawl", cache.number)
+            cache.crawl_deferred.callback(None)
+            return
+
+        # Do we know the chain length of the crawled peer? If not, make sure we get to know this first.
+        def on_latest_block(blocks):
+            if not blocks:
+                self.request_cache.pop(u"chaincrawl", cache.number)
+                cache.crawl_deferred.callback(None)
+                return
+
+            cache.known_chain_length = blocks[0].sequence_number
+            self.send_next_partial_chain_crawl_request(cache)
+
+        if not cache.known_chain_length:
+            self.send_crawl_request(cache.peer, cache.peer.public_key.key_to_bin(), -1, -1).addCallback(on_latest_block)
             return
 
         latest_block = self.persistence.get_latest(cache.peer.public_key.key_to_bin())
-        if not latest_block and cache.known_chain_length > 0:
-            # We have no knowledge of this peer, simply send a request from the genesis block to known chain length
+        if not latest_block:
+            # We have no knowledge of this peer but we have the length of the chain.
+            # Simply send a request from the genesis block to the known chain length.
             self.perform_partial_chain_crawl(cache, 1, cache.known_chain_length)
             return
         elif latest_block and lowest_unknown == latest_block.sequence_number + 1:
@@ -488,6 +518,7 @@ class TrustChainCommunity(Community):
                 return
             else:
                 self.request_cache.pop(u"chaincrawl", cache.number)
+                cache.crawl_deferred.callback(None)
                 return
 
         start, stop = self.persistence.get_lowest_range_unknown(cache.peer.public_key.key_to_bin())
@@ -605,38 +636,6 @@ class TrustChainCommunity(Community):
             self.logger.info("Received empty crawl response for crawl with ID %d", payload.crawl_id)
             cache.received_empty_response()
 
-    def get_trust(self, peer):
-        """
-        Return the trust score for a specific peer. For the basic Trustchain, this is the length of their chain.
-        """
-        block = self.persistence.get_latest(peer.public_key.key_to_bin())
-        if block:
-            return block.sequence_number
-        else:
-            # We need a minimum of 1 trust to have a chance to be selected in the categorical distribution.
-            return 1
-
-    def get_peer_for_introduction(self, exclude=None):
-        """
-        Choose a trusted peer to introduce you to someone else.
-        The more trust you have for someone, the higher the chance is to forward them.
-        """
-        eligible = [p for p in self.get_peers() if p != exclude]
-        if not eligible:
-            return super(TrustChainCommunity, self).get_peer_for_introduction(exclude)
-
-        total_trust = sum([self.get_trust(peer) for peer in eligible])
-        random_trust_i = random.randint(0, total_trust - 1)
-        current_trust_i = 0
-        for i in xrange(0, len(eligible)):
-            next_trust_i = self.get_trust(eligible[i])
-            if current_trust_i + next_trust_i > random_trust_i:
-                return eligible[i]
-            else:
-                current_trust_i += next_trust_i
-
-        return eligible[-1]
-
     def get_chain_length(self):
         """
         Return the length of your own chain.
